@@ -55,13 +55,13 @@ lws_ser_wu64be(uint8_t *b, uint64_t u64)
 uint16_t
 lws_ser_ru16be(const uint8_t *b)
 {
-	return (b[0] << 8) | b[1];
+	return (uint16_t)((b[0] << 8) | b[1]);
 }
 
 uint32_t
 lws_ser_ru32be(const uint8_t *b)
 {
-	return (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3];
+	return (unsigned int)((b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3]);
 }
 
 uint64_t
@@ -118,15 +118,15 @@ lws_vbi_decode(const void *buf, uint64_t *value, size_t len)
 signed char char_to_hex(const char c)
 {
 	if (c >= '0' && c <= '9')
-		return c - '0';
+		return (signed char)(c - '0');
 
 	if (c >= 'a' && c <= 'f')
-		return c - 'a' + 10;
+		return (signed char)(c - 'a' + 10);
 
 	if (c >= 'A' && c <= 'F')
-		return c - 'A' + 10;
+		return (signed char)(c - 'A' + 10);
 
-	return -1;
+	return (signed char)-1;
 }
 
 int
@@ -144,7 +144,7 @@ lws_hex_to_byte_array(const char *h, uint8_t *dest, int max)
 		if (t1 < 0)
 			return -1;
 
-		*dest++ = (t << 4) | t1;
+		*dest++ = (uint8_t)((t << 4) | t1);
 	}
 
 	if (max < 0)
@@ -155,21 +155,41 @@ lws_hex_to_byte_array(const char *h, uint8_t *dest, int max)
 
 static char *hexch = "0123456789abcdef";
 
+void
+lws_hex_from_byte_array(const uint8_t *src, size_t slen, char *dest, size_t len)
+{
+	char *end = &dest[len - 1];
+
+	while (slen-- && dest != end) {
+		uint8_t b = *src++;
+		*dest++ = hexch[b >> 4];
+		if (dest == end)
+			break;
+		*dest++ = hexch[b & 0xf];
+	}
+
+	*dest = '\0';
+}
+
 int
 lws_hex_random(struct lws_context *context, char *dest, size_t len)
 {
-	size_t n = (len - 1) / 2;
+	size_t n = ((len - 1) / 2) + 1;
 	uint8_t b, *r = (uint8_t *)dest + len - n;
 
 	if (lws_get_random(context, r, n) != n)
 		return 1;
 
-	while (n--) {
+	while (len >= 3) {
 		b = *r++;
 		*dest++ = hexch[b >> 4];
 		*dest++ = hexch[b & 0xf];
+		len -= 2;
 	}
 
+	if (len == 2)
+		*dest++ = hexch[(*r) >> 4];
+
 	*dest = '\0';
 
 	return 0;
@@ -254,7 +274,7 @@ lws_now_secs(void)
 
 	gettimeofday(&tv, NULL);
 
-	return tv.tv_sec;
+	return (unsigned long)tv.tv_sec;
 }
 
 #endif
@@ -453,7 +473,7 @@ lws_json_simple_find(const char *buf, size_t len, const char *name, size_t *alen
 		np++;
 	}
 
-	*alen = lws_ptr_diff(np, as);
+	*alen = (unsigned int)lws_ptr_diff(np, as);
 
 	return as;
 }
@@ -477,7 +497,7 @@ lws_json_simple_strcmp(const char *buf, size_t len, const char *name,
 static const char *hex = "0123456789ABCDEF";
 
 const char *
-lws_sql_purify(char *escaped, const char *string, int len)
+lws_sql_purify(char *escaped, const char *string, size_t len)
 {
 	const char *p = string;
 	char *q = escaped;
@@ -675,7 +695,7 @@ lws_urldecode(char *string, const char *escaped, int len)
 			if (n < 0)
 				return -1;
 			escaped++;
-			sum = n << 4;
+			sum = (char)(n << 4);
 			state++;
 			break;
 
@@ -684,7 +704,7 @@ lws_urldecode(char *string, const char *escaped, int len)
 			if (n < 0)
 				return -1;
 			escaped++;
-			*string++ = sum | n;
+			*string++ = (char)(sum | n);
 			len--;
 			state = 0;
 			break;
@@ -708,7 +728,7 @@ lws_finalize_startup(struct lws_context *context)
 
 #if !defined(LWS_PLAT_FREERTOS)
 void
-lws_get_effective_uid_gid(struct lws_context *context, int *uid, int *gid)
+lws_get_effective_uid_gid(struct lws_context *context, uid_t *uid, gid_t *gid)
 {
 	*uid = context->uid;
 	*gid = context->gid;
@@ -750,7 +770,7 @@ lws_timingsafe_bcmp(const void *a, const void *b, uint32_t len)
 	uint8_t sum = 0;
 
 	while (len--)
-		sum |= (*pa++ ^ *pb++);
+		sum |= (uint8_t)(*pa++ ^ *pb++);
 
 	return sum;
 }
@@ -768,8 +788,8 @@ lws_tokenize(struct lws_tokenize *ts)
 {
 	const char *rfc7230_delims = "(),/:;<=>?@[\\]{}";
 	lws_tokenize_state state = LWS_TOKZS_LEADING_WHITESPACE;
-	char c, flo = 0, d_minus = '-', d_dot = '.', s_minus = '\0',
-	     s_dot = '\0', skipping = 0;
+	char c, flo = 0, d_minus = '-', d_dot = '.', d_star = '*', s_minus = '\0',
+	     s_dot = '\0', s_star = '\0', d_eq = '=', s_eq = '\0', skipping = 0;
 	signed char num = (ts->flags & LWS_TOKENIZE_F_NO_INTEGERS) ? 0 : -1;
 	int utf8 = 0;
 
@@ -783,6 +803,14 @@ lws_tokenize(struct lws_tokenize *ts)
 		d_dot = '\0';
 		s_dot = '.';
 	}
+	if (ts->flags & LWS_TOKENIZE_F_ASTERISK_NONTERM) {
+		d_star = '\0';
+		s_star = '*';
+	}
+	if (ts->flags & LWS_TOKENIZE_F_EQUALS_NONTERM) {
+		d_eq = '\0';
+		s_eq = '=';
+	}
 
 	ts->token = NULL;
 	ts->token_len = 0;
@@ -791,7 +819,7 @@ lws_tokenize(struct lws_tokenize *ts)
 		c = *ts->start++;
 		ts->len--;
 
-		utf8 = lws_check_byte_utf8((unsigned char)utf8, c);
+		utf8 = lws_check_byte_utf8((unsigned char)utf8, (unsigned char)c);
 		if (utf8 < 0)
 			return LWS_TOKZE_ERR_BROKEN_UTF8;
 
@@ -856,7 +884,8 @@ lws_tokenize(struct lws_tokenize *ts)
 
 		/* token= aggregation */
 
-		if (c == '=' && (state == LWS_TOKZS_TOKEN_POST_TERMINAL ||
+		if (!(ts->flags & LWS_TOKENIZE_F_EQUALS_NONTERM) &&
+		    c == '=' && (state == LWS_TOKZS_TOKEN_POST_TERMINAL ||
 				 state == LWS_TOKZS_TOKEN)) {
 			if (num == 1)
 				return LWS_TOKZE_ERR_NUM_ON_LHS;
@@ -905,8 +934,8 @@ lws_tokenize(struct lws_tokenize *ts)
 		    ((!(ts->flags & LWS_TOKENIZE_F_RFC7230_DELIMS) &&
 		     (c < '0' || c > '9') && (c < 'A' || c > 'Z') &&
 		     (c < 'a' || c > 'z') && c != '_') &&
-		     c != s_minus && c != s_dot) ||
-		    c == d_minus || c == d_dot
+		     c != s_minus && c != s_dot && c != s_star && c != s_eq) ||
+		    c == d_minus || c == d_dot || c == d_star || c == d_eq
 		    ) &&
 		    !((ts->flags & LWS_TOKENIZE_F_SLASH_NONTERM) && c == '/')) {
 			switch (state) {
@@ -1019,7 +1048,7 @@ lws_tokenize_init(struct lws_tokenize *ts, const char *start, int flags)
 {
 	ts->start = start;
 	ts->len = 0x7fffffff;
-	ts->flags = flags;
+	ts->flags = (uint16_t)(unsigned int)flags;
 	ts->delim = LWSTZ_DT_NEED_FIRST_CONTENT;
 }
 
@@ -1135,6 +1164,82 @@ drain:
 	return LSTRX_DONE;
 }
 
+int
+lws_strcmp_wildcard(const char *wildcard, size_t len, const char *check)
+{
+	const char *match[3], *wc[3], *wc_end = wildcard + len;
+	int sp = 0;
+
+	do {
+
+		if (wildcard == wc_end) {
+			/*
+			 * We reached the end of wildcard, but not of check,
+			 * and the last thing in wildcard was not a * or we
+			 * would have completed already... if we can rewind,
+			 * let's try that...
+			 */
+			if (sp) {
+				wildcard = wc[sp - 1];
+				check = match[--sp];
+
+				continue;
+			}
+
+			/* otherwise it's the end of the road for this one */
+
+			return 1;
+		}
+
+		if (*wildcard == '*') {
+
+			if (++wildcard == wc_end)
+				 /*
+				  * Wildcard ended on a *, so we know we will
+				  * match unconditionally
+				  */
+				return 0;
+
+			/*
+			 * Now we need to stick wildcard here and see if there
+			 * is any remaining match exists, for eg b of "a*b"
+			 */
+
+			if (sp == LWS_ARRAY_SIZE(match)) {
+				lwsl_err("%s: exceeds * stack\n", __func__);
+				return 1; /* we can't deal with it */
+			}
+
+			wc[sp] = wildcard;
+			/* if we ever pop and come back here, pick up from +1 */
+			match[sp++] = check + 1;
+			continue;
+		}
+
+		if (*(check++) == *wildcard) {
+
+			if (wildcard == wc_end)
+				return 0;
+			/*
+			 * We're still compatible with wildcard... keep going
+			 */
+			wildcard++;
+
+			continue;
+		}
+
+		if (!sp)
+			/*
+			 * We're just trying to match literals, and failed...
+			 */
+			return 1;
+
+		/* we're looking for a post-* match... keep looking... */
+
+	} while (*check);
+
+	return !!*wildcard;
+}
 
 #if LWS_MAX_SMP > 1
 
@@ -1145,7 +1250,13 @@ lws_mutex_refcount_init(struct lws_mutex_refcount *mr)
 	mr->last_lock_reason = NULL;
 	mr->lock_depth = 0;
 	mr->metadata = 0;
+#ifdef __PTW32_H
+	/* If we use implementation of PThreads for Win that is
+	 * distributed by VCPKG */
+	memset(&mr->lock_owner, 0, sizeof(pthread_t));
+#else
 	mr->lock_owner = 0;
+#endif
 }
 
 void
@@ -1167,7 +1278,14 @@ lws_mutex_refcount_lock(struct lws_mutex_refcount *mr, const char *reason)
 	 *
 	 *  - it can be false and change to a different tid that is also false
 	 */
-	if (mr->lock_owner == pthread_self()) {
+#ifdef __PTW32_H
+	/* If we use implementation of PThreads for Win that is
+	 * distributed by VCPKG */
+	if (pthread_equal(mr->lock_owner, pthread_self()))
+#else
+	if (mr->lock_owner == pthread_self())
+#endif
+	{
 		/* atomic because we only change it if we own the lock */
 		mr->lock_depth++;
 		return;
@@ -1189,15 +1307,27 @@ lws_mutex_refcount_unlock(struct lws_mutex_refcount *mr)
 		return;
 
 	mr->last_lock_reason = "free";
+#ifdef __PTW32_H
+	/* If we use implementation of PThreads for Win that is
+	 * distributed by VCPKG */
+	memset(&mr->lock_owner, 0, sizeof(pthread_t));
+#else
 	mr->lock_owner = 0;
-	//lwsl_notice("tid %d: unlock %s\n", mr->tid, mr->last_lock_reason);
+#endif
+	// lwsl_notice("tid %d: unlock %s\n", mr->tid, mr->last_lock_reason);
 	pthread_mutex_unlock(&mr->lock);
 }
 
 void
 lws_mutex_refcount_assert_held(struct lws_mutex_refcount *mr)
 {
+#ifdef __PTW32_H
+	/* If we use implementation of PThreads for Win that is
+	 * distributed by VCPKG */
+	assert(pthread_equal(mr->lock_owner, pthread_self()) && mr->lock_depth);
+#else
 	assert(mr->lock_owner == pthread_self() && mr->lock_depth);
+#endif
 }
 
 #endif /* SMP */
@@ -1206,7 +1336,8 @@ lws_mutex_refcount_assert_held(struct lws_mutex_refcount *mr)
 const char *
 lws_cmdline_option(int argc, const char **argv, const char *val)
 {
-	int n = (int)strlen(val), c = argc;
+	size_t n = strlen(val);
+	int c = argc;
 
 	while (--c > 0) {
 
@@ -1229,13 +1360,18 @@ lws_cmdline_option(int argc, const char **argv, const char *val)
 
 static const char * const builtins[] = {
 	"-d",
-#if defined(LWS_WITH_UDP)
-	"--udp-tx-loss",
-	"--udp-rx-loss",
-#endif
+	"--fault-injection",
+	"--fault-seed",
 	"--ignore-sigterm"
 };
 
+enum opts {
+	OPT_DEBUGLEVEL,
+	OPT_FAULTINJECTION,
+	OPT_FAULT_SEED,
+	OPT_IGNORE_SIGTERM,
+};
+
 #if !defined(LWS_PLAT_FREERTOS)
 static void
 lws_sigterm_catch(int sig)
@@ -1249,6 +1385,9 @@ lws_cmdline_option_handle_builtin(int argc, const char **argv,
 {
 	const char *p;
 	int n, m, logs = LLL_USER | LLL_ERR | LLL_WARN | LLL_NOTICE;
+#if defined(LWS_WITH_SYS_FAULT_INJECTION)
+	uint64_t seed = (uint64_t)lws_now_usecs();
+#endif
 
 	for (n = 0; n < (int)LWS_ARRAY_SIZE(builtins); n++) {
 		p = lws_cmdline_option(argc, argv, builtins[n]);
@@ -1258,20 +1397,24 @@ lws_cmdline_option_handle_builtin(int argc, const char **argv,
 		m = atoi(p);
 
 		switch (n) {
-		case 0:
+		case OPT_DEBUGLEVEL:
 			logs = m;
 			break;
-#if defined(LWS_WITH_UDP)
-		case 1:
-			info->udp_loss_sim_tx_pc = m;
-			break;
-		case 2:
-			info->udp_loss_sim_rx_pc = m;
-			break;
-		case 3:
-#else
-		case 1:
+
+		case OPT_FAULTINJECTION:
+#if !defined(LWS_WITH_SYS_FAULT_INJECTION)
+			lwsl_err("%s: FAULT_INJECTION not built\n", __func__);
 #endif
+			lws_fi_deserialize(&info->fic, p);
+			break;
+
+		case OPT_FAULT_SEED:
+#if defined(LWS_WITH_SYS_FAULT_INJECTION)
+			seed = (uint64_t)atoll(p);
+#endif
+			break;
+
+		case OPT_IGNORE_SIGTERM:
 #if !defined(LWS_PLAT_FREERTOS)
 			signal(SIGTERM, lws_sigterm_catch);
 #endif
@@ -1279,77 +1422,92 @@ lws_cmdline_option_handle_builtin(int argc, const char **argv,
 		}
 	}
 
+#if defined(LWS_WITH_SYS_FAULT_INJECTION)
+	lws_xos_init(&info->fic.xos, seed);
+#endif
 	lws_set_log_level(logs, NULL);
+
+#if defined(LWS_WITH_SYS_FAULT_INJECTION)
+	if (info->fic.fi_owner.count)
+		lwsl_notice("%s: Fault Injection seed %llu\n", __func__,
+				(unsigned long long)seed);
+#endif
 }
 
 
 const lws_humanize_unit_t humanize_schema_si[] = {
-	{ "Pi ", LWS_PI }, { "Ti ", LWS_TI }, { "Gi ", LWS_GI },
-	{ "Mi ", LWS_MI }, { "Ki ", LWS_KI }, { "   ", 1 },
+	{ "Pi", LWS_PI }, { "Ti", LWS_TI }, { "Gi", LWS_GI },
+	{ "Mi", LWS_MI }, { "Ki", LWS_KI }, { "", 1 },
 	{ NULL, 0 }
 };
 const lws_humanize_unit_t humanize_schema_si_bytes[] = {
 	{ "PiB", LWS_PI }, { "TiB", LWS_TI }, { "GiB", LWS_GI },
-	{ "MiB", LWS_MI }, { "KiB", LWS_KI }, { "B  ", 1 },
+	{ "MiB", LWS_MI }, { "KiB", LWS_KI }, { "B", 1 },
 	{ NULL, 0 }
 };
 const lws_humanize_unit_t humanize_schema_us[] = {
-	{ "y  ",  (uint64_t)365 * 24 * 3600 * LWS_US_PER_SEC },
-	{ "d  ",  (uint64_t)24 * 3600 * LWS_US_PER_SEC },
-	{ "hr ", (uint64_t)3600 * LWS_US_PER_SEC },
+	{ "y",  (uint64_t)365 * 24 * 3600 * LWS_US_PER_SEC },
+	{ "d",  (uint64_t)24 * 3600 * LWS_US_PER_SEC },
+	{ "hr", (uint64_t)3600 * LWS_US_PER_SEC },
 	{ "min", 60 * LWS_US_PER_SEC },
-	{ "s  ", LWS_US_PER_SEC },
-	{ "ms ", LWS_US_PER_MS },
-	{ "us ", 1 },
+	{ "s", LWS_US_PER_SEC },
+	{ "ms", LWS_US_PER_MS },
+#if defined(WIN32)
+	{ "us", 1 },
+#else
+	{ "μs", 1 },
+#endif
 	{ NULL, 0 }
 };
 
+/* biggest ull is 18446744073709551615 (20 chars) */
+
 static int
 decim(char *r, uint64_t v, char chars, char leading)
 {
-	int n = chars - 1;
 	uint64_t q = 1;
+	char *ro = r;
+	int n = 1;
 
-	r += n;
-
-	while (n >= 0) {
-		if (v / q)
-			*r-- = '0' + ((v / q) % 10);
-		else
-			*r-- = leading ? '0' : ' ';
+	while ((leading || v > (q * 10) - 1) && n < 20 && n < chars) {
 		q = q * 10;
-		n--;
+		n++;
 	}
 
-	if (v / q)
-		/* the number is bigger than the allowed chars! */
-		r[1] = '!';
+	/* n is how many chars needed */
 
-	return chars;
+	while (n--) {
+		*r++ = (char)('0' + (char)((v / q) % 10));
+		q = q / 10;
+	}
+
+	*r = '\0';
+
+	return lws_ptr_diff(r, ro);
 }
 
 int
-lws_humanize(char *p, int len, uint64_t v, const lws_humanize_unit_t *schema)
+lws_humanize(char *p, size_t len, uint64_t v, const lws_humanize_unit_t *schema)
 {
-	char *end = p + len;
+	char *obuf = p, *end = p + len;
 
 	do {
 		if (v >= schema->factor || schema->factor == 1) {
 			if (schema->factor == 1) {
-				*p++ = ' ';
 				p += decim(p, v, 4, 0);
-				return lws_snprintf(p, lws_ptr_diff(end, p),
-						    "%s    ", schema->name);
+				p += lws_snprintf(p, lws_ptr_diff_size_t(end, p),
+						    "%s", schema->name);
+				return lws_ptr_diff(p, obuf);
 			}
 
-			*p++ = ' ';
 			p += decim(p, v / schema->factor, 4, 0);
 			*p++ = '.';
 			p += decim(p, (v % schema->factor) /
 					(schema->factor / 1000), 3, 1);
 
-			return lws_snprintf(p, lws_ptr_diff(end, p),
+			p += lws_snprintf(p, lws_ptr_diff_size_t(end, p),
 					    "%s", schema->name);
+			return lws_ptr_diff(p, obuf);
 		}
 		schema++;
 	} while (schema->name);
