@@ -87,11 +87,7 @@ lws_plat_init(struct lws_context *context,
 	}
 
 	while (n--) {
-               int m;
 		pt->fds_count = 0;
-               for (m = 0; m < WSA_MAXIMUM_WAIT_EVENTS; m++)
-                       pt->events[m] = WSACreateEvent();
-		InitializeCriticalSection(&pt->interrupt_lock);
 
 		pt++;
 	}
@@ -111,16 +107,7 @@ lws_plat_init(struct lws_context *context,
 void
 lws_plat_context_early_destroy(struct lws_context *context)
 {
-	struct lws_context_per_thread *pt = &context->pt[0];
-	int n = context->count_threads;
 
-	while (n--) {
-		int m;
-		for (m = 0; m < WSA_MAXIMUM_WAIT_EVENTS; m++)
-			WSACloseEvent(pt->events[m]);
-		DeleteCriticalSection(&pt->interrupt_lock);
-		pt++;
-	}
 }
 
 void
