@@ -90,7 +90,8 @@ const struct http2_settings lws_h2_stock_settings = { {
 }};
 
 /*
- * The wsi at this level is the network wsi
+ * The wsi at this level is normally the network wsi... we can get called on
+ * another path via lws_service_do_ripe_rxflow() on mux children too tho...
  */
 
 static int
@@ -132,7 +133,7 @@ rops_handle_POLLIN_h2(struct lws_context_per_thread *pt, struct lws *wsi,
 			return LWS_HPI_RET_PLEASE_CLOSE_ME;
 		}
 
-		n = lws_client_socket_service(wsi, pollfd);
+		n = lws_http_client_socket_service(wsi, pollfd);
 		if (n)
 			return LWS_HPI_RET_WSI_ALREADY_DIED;
 #endif
@@ -190,6 +191,14 @@ read:
 		lwsl_info("draining buflist (len %d)\n", ebuf.len);
 		buffered = 1;
 		goto drain;
+	} else {
+
+		if (wsi->mux_substream) {
+			lwsl_warn("%s: uh... %s mux child with nothing to drain\n", __func__, lws_wsi_tag(wsi));
+			// assert(0);
+			lws_dll2_remove(&wsi->dll_buflist);
+			return LWS_HPI_RET_HANDLED;
+		}
 	}
 
 	if (!lws_ssl_pending(wsi) &&
@@ -265,9 +274,9 @@ drain:
 		n = 0;
 		if (lwsi_role_h2(wsi) && lwsi_state(wsi) != LRS_BODY &&
 		    lwsi_state(wsi) != LRS_DISCARD_BODY)
-			n = lws_read_h2(wsi, ebuf.token, ebuf.len);
+			n = lws_read_h2(wsi, ebuf.token, (unsigned int)ebuf.len);
 		else
-			n = lws_read_h1(wsi, ebuf.token, ebuf.len);
+			n = lws_read_h1(wsi, ebuf.token, (unsigned int)ebuf.len);
 
 		if (n < 0) {
 			/* we closed wsi */
@@ -280,21 +289,21 @@ drain:
 			lwsl_info("%s: draining rxflow: used %d, next %d\n",
 				    __func__, n, m);
 			if (!m) {
-				lwsl_notice("%s: removed %p from dll_buflist\n",
-					    __func__, wsi);
+				lwsl_notice("%s: removed %s from dll_buflist\n",
+					    __func__, lws_wsi_tag(wsi));
 				lws_dll2_remove(&wsi->dll_buflist);
 			}
 		} else
-			if (n && n != ebuf.len) {
+			if (n && n < ebuf.len && ebuf.len > 0) {
 				// lwsl_notice("%s: h2 append seg %d\n", __func__, ebuf.len - n);
 				m = lws_buflist_append_segment(&wsi->buflist,
 						ebuf.token + n,
-						ebuf.len - n);
+						(unsigned int)(ebuf.len - n));
 				if (m < 0)
 					return LWS_HPI_RET_PLEASE_CLOSE_ME;
 				if (m) {
-					lwsl_debug("%s: added %p to rxflow list\n",
-							__func__, wsi);
+					lwsl_debug("%s: added %s to rxflow list\n",
+						   __func__, lws_wsi_tag(wsi));
 					if (lws_dll2_is_detached(&wsi->dll_buflist))
 						lws_dll2_add_head(&wsi->dll_buflist,
 							 &pt->dll_buflist_owner);
@@ -323,7 +332,7 @@ drain:
 	}
 #endif
 
-	pending = lws_ssl_pending(wsi);
+	pending = (unsigned int)lws_ssl_pending(wsi);
 	if (pending) {
 		// lwsl_info("going around\n");
 		goto read;
@@ -420,9 +429,9 @@ rops_write_role_protocol_h2(struct lws *wsi, unsigned char *buf, size_t len,
 		if (n)
 			return n;
 
-		lwsl_info("%s: %p: transformed %d bytes to %d "
+		lwsl_info("%s: %s: transformed %d bytes to %d "
 			   "(wp 0x%x, more %d)\n", __func__,
-			   wsi, (int)len, (int)o, (int)*wp,
+			   lws_wsi_tag(wsi), (int)len, (int)o, (int)*wp,
 			   wsi->http.comp_ctx.may_have_more);
 
 		buf = out;
@@ -430,7 +439,7 @@ rops_write_role_protocol_h2(struct lws *wsi, unsigned char *buf, size_t len,
 		base = (*wp) & 0x1f;
 
 		if (!len)
-			return olen;
+			return (int)olen;
 	}
 #endif
 
@@ -465,7 +474,8 @@ rops_write_role_protocol_h2(struct lws *wsi, unsigned char *buf, size_t len,
 	     base == LWS_WRITE_HTTP_FINAL) &&
 	     wsi->http.tx_content_length) {
 		wsi->http.tx_content_remain -= len;
-		lwsl_info("%s: wsi %p: tx_content_rem = %llu\n", __func__, wsi,
+		lwsl_info("%s: %s: tx_content_rem = %llu\n", __func__,
+			  lws_wsi_tag(wsi),
 			  (unsigned long long)wsi->http.tx_content_remain);
 		if (!wsi->http.tx_content_remain) {
 			lwsl_info("%s: selecting final write mode\n", __func__);
@@ -474,12 +484,13 @@ rops_write_role_protocol_h2(struct lws *wsi, unsigned char *buf, size_t len,
 	}
 
 	if (base == LWS_WRITE_HTTP_FINAL || ((*wp) & LWS_WRITE_H2_STREAM_END)) {
-		lwsl_info("%s: %p: setting END_STREAM\n", __func__, wsi);
+		lwsl_info("%s: %s: setting END_STREAM\n", __func__,
+				lws_wsi_tag(wsi));
 		flags |= LWS_H2_FLAG_END_STREAM;
 		wsi->h2.send_END_STREAM = 1;
 	}
 
-	n = lws_h2_frame_write(wsi, n, flags, wsi->mux.my_sid, (int)len, buf);
+	n = lws_h2_frame_write(wsi, n, flags, wsi->mux.my_sid, (unsigned int)len, buf);
 	if (n < 0)
 		return n;
 
@@ -510,13 +521,12 @@ rops_check_upgrades_h2(struct lws *wsi)
 	if (!p || strcmp(p, "websocket"))
 		return LWS_UPG_RET_CONTINUE;
 
-#if defined(LWS_WITH_SERVER_STATUS)
-	wsi->a.vhost->conn_stats.ws_upg++;
-#endif
 	lwsl_info("Upgrade h2 to ws\n");
 	lws_mux_mark_immortal(wsi);
 	wsi->h2_stream_carries_ws = 1;
 
+	lws_metrics_tag_wsi_add(wsi, "upg", "ws_over_h2");
+
 	if (lws_process_ws_upgrade(wsi))
 		return LWS_UPG_RET_BAIL;
 
@@ -582,7 +592,7 @@ rops_tx_credit_h2(struct lws *wsi, char peer_to_us, int add)
 			 * We want to tell the peer they can write an additional
 			 * "add" bytes to us
 			 */
-			return lws_h2_update_peer_txcredit(wsi, -1, add);
+			return lws_h2_update_peer_txcredit(wsi, (unsigned int)-1, add);
 		}
 
 		/*
@@ -613,14 +623,14 @@ rops_destroy_role_h2(struct lws *wsi)
 	struct allocated_headers *ah;
 
 	/* we may not have an ah, but may be on the waiting list... */
-	lwsl_info("%s: wsi %p: ah det due to close\n", __func__, wsi);
+	lwsl_info("%s: %s: ah det due to close\n", __func__, lws_wsi_tag(wsi));
 	__lws_header_table_detach(wsi, 0);
 
 	ah = pt->http.ah_list;
 
 	while (ah) {
 		if (ah->in_use && ah->wsi == wsi) {
-			lwsl_err("%s: ah leak: wsi %p\n", __func__, wsi);
+			lwsl_err("%s: ah leak: %s\n", __func__, lws_wsi_tag(wsi));
 			ah->in_use = 0;
 			ah->wsi = NULL;
 			pt->http.ah_count_in_use--;
@@ -667,7 +677,7 @@ rops_close_kill_connection_h2(struct lws *wsi, enum lws_close_status reason)
 			lws_h2_rst_stream(wsi, H2_ERR_STREAM_CLOSED, "swsi got closed");
 */
 
-	lwsl_info(" wsi: %p, his parent %p: siblings:\n", wsi, wsi->mux.parent_wsi);
+	lwsl_info(" %s, his parent %s: siblings:\n", lws_wsi_tag(wsi), lws_wsi_tag(wsi->mux.parent_wsi));
 	lws_wsi_mux_dump_children(wsi);
 
 	if (wsi->upgraded_to_http2 || wsi->mux_substream
@@ -675,13 +685,14 @@ rops_close_kill_connection_h2(struct lws *wsi, enum lws_close_status reason)
 			|| wsi->client_mux_substream
 #endif
 	) {
-		lwsl_info("closing %p: parent %p\n", wsi, wsi->mux.parent_wsi);
+		lwsl_info("closing %s: parent %s\n", lws_wsi_tag(wsi),
+				lws_wsi_tag(wsi->mux.parent_wsi));
 
 		if (wsi->mux.child_list && lwsl_visible(LLL_INFO)) {
-			lwsl_info(" parent %p: closing children: list:\n", wsi);
+			lwsl_info(" parent %s: closing children: list:\n", lws_wsi_tag(wsi));
 			lws_wsi_mux_dump_children(wsi);
 		}
-		lws_wsi_mux_close_children(wsi, reason);
+		lws_wsi_mux_close_children(wsi, (int)reason);
 	}
 
 	if (wsi->upgraded_to_http2) {
@@ -764,51 +775,108 @@ rops_callback_on_writable_h2(struct lws *wsi)
 static int
 lws_h2_bind_for_post_before_action(struct lws *wsi)
 {
+	const struct lws_http_mount *hit;
+	char *uri_ptr = NULL;
+	uint8_t *buffered;
+	int uri_len = 0;
 	const char *p;
+	size_t blen;
 
 	p = lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_METHOD);
-	if (p && !strcmp(p, "POST")) {
-		const struct lws_http_mount *hit;
+	if (!p || strcmp(p, "POST"))
+		return 0;
 
-		if (!lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_COLON_PATH) ||
-		    !lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH))
-			/*
-			 * There must be a path.  Actually this is checked at
-			 * http2.c along with the other required header
-			 * presence before we can get here.
-			 *
-			 * But Coverity insists to see us check it.
-			 */
+
+	if (!lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_COLON_PATH) ||
+	    !lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH))
+		/*
+		 * There must be a path.  Actually this is checked at
+		 * http2.c along with the other required header
+		 * presence before we can get here.
+		 *
+		 * But Coverity insists to see us check it.
+		 */
+		return 1;
+
+	hit = lws_find_mount(wsi,
+		  lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH),
+		  lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_COLON_PATH));
+
+	lwsl_debug("%s: %s: hit %p: %s\n", __func__,
+		    lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH),
+		    hit, hit ? hit->origin : "null");
+	if (hit) {
+		const struct lws_protocols *pp;
+		const char *name = hit->origin;
+
+		if (hit->origin_protocol == LWSMPRO_CGI ||
+		    hit->origin_protocol == LWSMPRO_HTTP ||
+		    hit->origin_protocol == LWSMPRO_HTTPS)
+			return 0;
+
+		if (hit->protocol)
+			name = hit->protocol;
+
+		pp = lws_vhost_name_to_protocol(wsi->a.vhost, name);
+		if (!pp) {
+			lwsl_info("Unable to find protocol '%s'\n", name);
 			return 1;
-
-		hit = lws_find_mount(wsi,
-			  lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH),
-			  lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_COLON_PATH));
-
-		lwsl_debug("%s: %s: hit %p: %s\n", __func__,
-			    lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_COLON_PATH),
-			    hit, hit ? hit->origin : "null");
-		if (hit) {
-			const struct lws_protocols *pp;
-			const char *name = hit->origin;
-
-			if (hit->protocol)
-				name = hit->protocol;
-
-			pp = lws_vhost_name_to_protocol(wsi->a.vhost, name);
-			if (!pp) {
-				lwsl_info("Unable to find protocol '%s'\n", name);
-				return 1;
-			}
-
-			if (lws_bind_protocol(wsi, pp, __func__))
-				return 1;
 		}
 
-		lwsl_info("%s: setting LRS_BODY from 0x%x (%s)\n", __func__,
-			    (int)wsi->wsistate, wsi->a.protocol->name);
-		lwsi_set_state(wsi, LRS_BODY);
+		if (lws_bind_protocol(wsi, pp, __func__))
+			return 1;
 	}
+	if (lws_http_get_uri_and_method(wsi, &uri_ptr, &uri_len) >= 0)
+		if (wsi->a.protocol->callback(wsi, LWS_CALLBACK_HTTP,
+					      wsi->user_space,
+					      hit ? uri_ptr +
+						  hit->mountpoint_len : uri_ptr,
+					      (size_t)(hit ? uri_len -
+							  hit->mountpoint_len :
+							  uri_len)))
+			return 1;
+
+	lwsl_info("%s: setting LRS_BODY from 0x%x (%s)\n", __func__,
+		    (int)wsi->wsistate, wsi->a.protocol->name);
+
+	lwsi_set_state(wsi, LRS_BODY);
+
+	if (wsi->http.content_length_explicitly_zero)
+		return 0;
+
+	/*
+	 * Dump any stashed body
+	 */
+
+	while (((!wsi->http.content_length_given) ||
+		  wsi->http.rx_content_length) &&
+	       (blen = lws_buflist_next_segment_len(&wsi->buflist, &buffered))) {
+
+		if ((size_t)wsi->http.rx_content_length < blen)
+			blen = (size_t)wsi->http.rx_content_length;
+
+		if (wsi->a.protocol->callback(wsi, LWS_CALLBACK_HTTP_BODY,
+				wsi->user_space, buffered, blen))
+			return 1;
+		lws_buflist_use_segment(&wsi->buflist, blen);
+
+		wsi->http.rx_content_length -= blen;
+	}
+
+	if (!wsi->buflist)
+		/* Take us off the pt's "wsi holding input buflist" list */
+		lws_dll2_remove(&wsi->dll_buflist);
+
+	if (wsi->http.content_length_given && wsi->http.rx_content_length)
+		/* still a-ways to go */
+		return 0;
+
+	if (!wsi->http.content_length_given && !wsi->h2.END_STREAM)
+		return 0;
+
+	if (wsi->a.protocol->callback(wsi, LWS_CALLBACK_HTTP_BODY_COMPLETION,
+				      wsi->user_space, NULL, 0))
+		return 1;
 
 	return 0;
 }
@@ -862,7 +930,7 @@ rops_perform_user_POLLOUT_h2(struct lws *wsi)
 		 * move him to be the last child
 		 */
 
-		lwsl_debug("servicing child %p\n", *wsi2);
+		lwsl_debug("servicing child %s\n", lws_wsi_tag(*wsi2));
 
 		w = lws_wsi_mux_move_child_to_tail(wsi2);
 
@@ -871,8 +939,9 @@ rops_perform_user_POLLOUT_h2(struct lws *wsi)
 			goto next_child;
 		}
 
-		lwsl_info("%s: child wsi %p, sid %d, (wsistate 0x%x)\n",
-			  __func__, w, w->mux.my_sid, (unsigned int)w->wsistate);
+		lwsl_info("%s: child %s, sid %d, (wsistate 0x%x)\n",
+			  __func__, lws_wsi_tag(w), w->mux.my_sid,
+			  (unsigned int)w->wsistate);
 
 		/* priority 1: post compression-transform buffered output */
 
@@ -961,7 +1030,31 @@ rops_perform_user_POLLOUT_h2(struct lws *wsi)
 
 			lwsi_set_state(w, LRS_ESTABLISHED);
 
-			lws_h2_bind_for_post_before_action(w);
+			if (w->buflist) {
+				struct lws_context_per_thread *pt;
+
+				pt = &w->a.context->pt[(int)w->tsi];
+				lwsl_debug("%s: added %s to rxflow list\n",
+					   __func__, lws_wsi_tag(w));
+				lws_dll2_add_head(
+					&w->dll_buflist,
+					&pt->dll_buflist_owner);
+			}
+
+			if (lws_h2_bind_for_post_before_action(w))
+				return -1;
+
+			/*
+			 * Well, we could be getting a POST from the client, it
+			 * may not have any content-length.  In that case, we
+			 * will be in LRS_BODY state, we can't actually start
+			 * the action until we had the body and the stream is
+			 * half-closed, indicating that we can reply
+			 */
+
+			if (lwsi_state(w) == LRS_BODY &&
+			    w->h2.h2_state != LWS_H2_STATE_HALF_CLOSED_REMOTE)
+				goto next_child;
 
 			lwsl_info("  h2 action start...\n");
 			n = lws_http_action(w);
@@ -1018,7 +1111,8 @@ rops_perform_user_POLLOUT_h2(struct lws *wsi)
 			 * DATA here... if so close the actual wsi
 			 */
 			if (n < 0 || w->h2.send_END_STREAM) {
-				lwsl_debug("Closing POLLOUT child %p\n", w);
+				lwsl_debug("Closing POLLOUT child %s\n",
+						lws_wsi_tag(w));
 				lws_close_free_wsi(w, LWS_CLOSE_STATUS_NOSTATUS,
 						   "h2 end stream file");
 				wa = &wsi->mux.child_list;
@@ -1071,7 +1165,7 @@ rops_perform_user_POLLOUT_h2(struct lws *wsi)
 					     LWS_WRITE_H2_STREAM_END;
 
 			n = lws_write(w, &w->ws->ping_payload_buf[LWS_PRE],
-				      w->ws->ping_payload_len, write_type);
+				      w->ws->ping_payload_len, (enum lws_write_protocol)write_type);
 			if (n < 0)
 				return -1;
 
@@ -1110,12 +1204,12 @@ rops_perform_user_POLLOUT_h2(struct lws *wsi)
 
 			if (!rops_write_role_protocol_h2(w, buf + LWS_PRE, 0,
 							 &wp)) {
-				lwsl_info("%s: wsi %p: entering ro long poll\n",
-					  __func__, w);
+				lwsl_info("%s: %s: entering ro long poll\n",
+					  __func__, lws_wsi_tag(w));
 				lws_mux_mark_immortal(w);
 			} else
-				lwsl_err("%s: wsi %p: failed to set long poll\n",
-						__func__, w);
+				lwsl_err("%s: %s: failed to set long poll\n",
+						__func__, lws_wsi_tag(w));
 			goto next_child;
 		}
 
@@ -1164,15 +1258,12 @@ rops_alpn_negotiated_h2(struct lws *wsi, const char *alpn)
 #endif
 
 	wsi->upgraded_to_http2 = 1;
-#if defined(LWS_WITH_SERVER_STATUS)
-	wsi->a.vhost->conn_stats.h2_alpn++;
-#endif
 
 	/* adopt the header info */
 
 	ah = wsi->http.ah;
 
-	lws_role_transition(wsi, LWSIFR_SERVER, LRS_H2_AWAIT_PREFACE,
+	lws_role_transition(wsi, lwsi_role_client(wsi) ? LWSIFR_CLIENT : LWSIFR_SERVER, LRS_H2_AWAIT_PREFACE,
 			    &role_ops_h2);
 
 	/* http2 union member has http union struct at start */
@@ -1188,10 +1279,10 @@ rops_alpn_negotiated_h2(struct lws *wsi, const char *alpn)
 	/* HTTP2 union */
 
 	lws_hpack_dynamic_size(wsi,
-			   wsi->h2.h2n->our_set.s[H2SET_HEADER_TABLE_SIZE]);
+			   (int)wsi->h2.h2n->our_set.s[H2SET_HEADER_TABLE_SIZE]);
 	wsi->txc.tx_cr = 65535;
 
-	lwsl_info("%s: wsi %p: configured for h2\n", __func__, wsi);
+	lwsl_info("%s: %s: configured for h2\n", __func__, lws_wsi_tag(wsi));
 
 	return 0;
 }
@@ -1201,7 +1292,7 @@ rops_issue_keepalive_h2(struct lws *wsi, int isvalid)
 {
 	struct lws *nwsi = lws_get_network_wsi(wsi);
 	struct lws_h2_protocol_send *pps;
-	uint64_t us = lws_now_usecs();
+	uint64_t us = (uint64_t)lws_now_usecs();
 
 	if (isvalid) {
 		_lws_validity_confirmed_role(nwsi);
@@ -1234,33 +1325,59 @@ rops_issue_keepalive_h2(struct lws *wsi, int isvalid)
 	return 0;
 }
 
+static const lws_rops_t rops_table_h2[] = {
+#if defined(LWS_WITH_SERVER)
+	/*  1 */ { .check_upgrades	  = rops_check_upgrades_h2 },
+#else
+	/*  1 */ { .check_upgrades	  = NULL },
+#endif
+	/*  2 */ { .pt_init_destroy	  = rops_pt_init_destroy_h2 },
+	/*  3 */ { .init_vhost		  = rops_init_vhost_h2 },
+	/*  4 */ { .handle_POLLIN	  = rops_handle_POLLIN_h2 },
+	/*  5 */ { .handle_POLLOUT	  = rops_handle_POLLOUT_h2 },
+	/*  6 */ { .perform_user_POLLOUT  = rops_perform_user_POLLOUT_h2 },
+	/*  7 */ { .callback_on_writable  = rops_callback_on_writable_h2 },
+	/*  8 */ { .tx_credit		  = rops_tx_credit_h2 },
+	/*  9 */ { .write_role_protocol	  = rops_write_role_protocol_h2 },
+	/* 10 */ { .encapsulation_parent  = rops_encapsulation_parent_h2 },
+	/* 11 */ { .alpn_negotiated	  = rops_alpn_negotiated_h2 },
+	/* 12 */ { .close_kill_connection = rops_close_kill_connection_h2 },
+	/* 13 */ { .destroy_role	  = rops_destroy_role_h2 },
+	/* 14 */ { .issue_keepalive	  = rops_issue_keepalive_h2 },
+};
+
+
 const struct lws_role_ops role_ops_h2 = {
 	/* role name */			"h2",
 	/* alpn id */			"h2",
+
+	/* rops_table */		rops_table_h2,
+	/* rops_idx */			{
+	  /* LWS_ROPS_check_upgrades */
 #if defined(LWS_WITH_SERVER)
-	/* check_upgrades */		rops_check_upgrades_h2,
+	  /* LWS_ROPS_pt_init_destroy */		0x12,
 #else
-					NULL,
+	  /* LWS_ROPS_pt_init_destroy */		0x02,
 #endif
-	/* pt_init_destroy */		rops_pt_init_destroy_h2,
-	/* init_vhost */		rops_init_vhost_h2,
-	/* destroy_vhost */		NULL,
-	/* service_flag_pending */	NULL,
-	/* handle_POLLIN */		rops_handle_POLLIN_h2,
-	/* handle_POLLOUT */		rops_handle_POLLOUT_h2,
-	/* perform_user_POLLOUT */	rops_perform_user_POLLOUT_h2,
-	/* callback_on_writable */	rops_callback_on_writable_h2,
-	/* tx_credit */			rops_tx_credit_h2,
-	/* write_role_protocol */	rops_write_role_protocol_h2,
-	/* encapsulation_parent */	rops_encapsulation_parent_h2,
-	/* alpn_negotiated */		rops_alpn_negotiated_h2,
-	/* close_via_role_protocol */	NULL,
-	/* close_role */		NULL,
-	/* close_kill_connection */	rops_close_kill_connection_h2,
-	/* destroy_role */		rops_destroy_role_h2,
-	/* adoption_bind */		NULL,
-	/* client_bind */		NULL,
-	/* issue_keepalive */		rops_issue_keepalive_h2,
+	  /* LWS_ROPS_init_vhost */
+	  /* LWS_ROPS_destroy_vhost */			0x30,
+	  /* LWS_ROPS_service_flag_pending */
+	  /* LWS_ROPS_handle_POLLIN */			0x04,
+	  /* LWS_ROPS_handle_POLLOUT */
+	  /* LWS_ROPS_perform_user_POLLOUT */		0x56,
+	  /* LWS_ROPS_callback_on_writable */
+	  /* LWS_ROPS_tx_credit */			0x78,
+	  /* LWS_ROPS_write_role_protocol */
+	  /* LWS_ROPS_encapsulation_parent */		0x9a,
+	  /* LWS_ROPS_alpn_negotiated */
+	  /* LWS_ROPS_close_via_role_protocol */	0xb0,
+	  /* LWS_ROPS_close_role */
+	  /* LWS_ROPS_close_kill_connection */		0x0c,
+	  /* LWS_ROPS_destroy_role */
+	  /* LWS_ROPS_adoption_bind */			0xd0,
+	  /* LWS_ROPS_client_bind */
+	  /* LWS_ROPS_issue_keepalive */		0x0e,
+					},
 	/* adoption_cb clnt, srv */	{ LWS_CALLBACK_SERVER_NEW_CLIENT_INSTANTIATED,
 					  LWS_CALLBACK_SERVER_NEW_CLIENT_INSTANTIATED },
 	/* rx cb clnt, srv */		{ LWS_CALLBACK_RECEIVE_CLIENT_HTTP,
