@@ -94,8 +94,8 @@ _lws_vhost_init_server(const struct lws_context_creation_info *info,
 		 * let's check before we do anything else about the disposition
 		 * of the interface he wants to bind to...
 		 */
-		is = lws_socket_bind(vhost, LWS_SOCK_INVALID, vhost->listen_port,
-				vhost->iface, 1);
+		is = lws_socket_bind(vhost, NULL, LWS_SOCK_INVALID,
+				     vhost->listen_port, vhost->iface, 1);
 		lwsl_debug("initial if check says %d\n", is);
 
 		if (is == LWS_ITOSA_BUSY)
@@ -175,17 +175,23 @@ done_list:
 #endif
 
 	for (m = 0; m < limit; m++) {
+
+		if (lws_fi(&vhost->fic, "listenskt")) {
+			sockfd = LWS_SOCK_INVALID;
+		} else {
+
 #ifdef LWS_WITH_UNIX_SOCK
-		if (LWS_UNIX_SOCK_ENABLED(vhost))
-			sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
-		else
+			if (LWS_UNIX_SOCK_ENABLED(vhost))
+				sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
+			else
 #endif
 #ifdef LWS_WITH_IPV6
-		if (LWS_IPV6_ENABLED(vhost))
-			sockfd = socket(AF_INET6, SOCK_STREAM, 0);
-		else
+			if (LWS_IPV6_ENABLED(vhost))
+				sockfd = socket(AF_INET6, SOCK_STREAM, 0);
+			else
 #endif
-			sockfd = socket(AF_INET, SOCK_STREAM, 0);
+				sockfd = socket(AF_INET, SOCK_STREAM, 0);
+		}
 
 		if (sockfd == LWS_SOCK_INVALID) {
 			lwsl_err("ERROR opening socket\n");
@@ -252,7 +258,7 @@ done_list:
 #endif
 		lws_plat_set_socket_options(vhost, sockfd, 0);
 
-		is = lws_socket_bind(vhost, sockfd, vhost->listen_port,
+		is = lws_socket_bind(vhost, NULL, sockfd, vhost->listen_port,
 				     vhost->iface, 1);
 		if (is == LWS_ITOSA_BUSY) {
 			/* treat as fatal */
@@ -272,21 +278,16 @@ done_list:
 			goto deal;
 		}
 
-		{
-			size_t s = sizeof(struct lws);
+		/*
+		 * Create the listen wsi and customize it
+		 */
 
-#if defined(LWS_WITH_EVENT_LIBS)
-			s += vhost->context->event_loop_ops->evlib_size_wsi;
-#endif
-
-			wsi = lws_zalloc(s, "listen wsi");
-			if (wsi == NULL) {
-				lwsl_err("Out of mem\n");
-				goto bail;
-			}
-#if defined(LWS_WITH_EVENT_LIBS)
-			wsi->evlib_wsi = (uint8_t *)wsi + sizeof(*wsi);
-#endif
+		lws_context_lock(vhost->context, __func__);
+		wsi = __lws_wsi_create_with_role(vhost->context, m, &role_ops_listen);
+		lws_context_unlock(vhost->context);
+		if (wsi == NULL) {
+			lwsl_err("Out of mem\n");
+			goto bail;
 		}
 
 #ifdef LWS_WITH_UNIX_SOCK
@@ -299,11 +300,8 @@ done_list:
 			lwsl_debug("%s: lws_socket_bind says %d\n", __func__, is);
 		}
 
-		wsi->a.context = vhost->context;
 		wsi->desc.sockfd = sockfd;
-		lws_role_transition(wsi, 0, LRS_UNCONNECTED, &role_ops_listen);
 		wsi->a.protocol = vhost->protocols;
-		wsi->tsi = m;
 		lws_vhost_bind_wsi(vhost, wsi);
 		wsi->listener = 1;
 
@@ -319,7 +317,6 @@ done_list:
 			goto bail;
 		}
 
-		vhost->context->count_wsi_allocated++;
 		vhost->lserv_wsi = wsi;
 		lws_pt_unlock(pt);
 
@@ -327,10 +324,16 @@ done_list:
 		if (n < 0) {
 			lwsl_err("listen failed with error %d\n", LWS_ERRNO);
 			vhost->lserv_wsi = NULL;
-			vhost->context->count_wsi_allocated--;
 			__remove_wsi_socket_from_fds(wsi);
 			goto bail;
 		}
+
+		if (wsi)
+			__lws_lc_tag(&vhost->context->lcg[LWSLCG_WSI],
+				     &wsi->lc, "listen|%s|%s|%d", vhost->name,
+				     vhost->iface ? vhost->iface : "",
+				     (int)vhost->listen_port);
+
 	} /* for each thread able to independently listen */
 
 	if (!lws_check_opt(vhost->context->options,
@@ -371,7 +374,7 @@ lws_select_vhost(struct lws_context *context, int port, const char *servername)
 
 	while (vhost) {
 		if (port == vhost->listen_port &&
-		    !strncmp(vhost->name, servername, colon)) {
+		    !strncmp(vhost->name, servername, (unsigned int)colon)) {
 			lwsl_info("SNI: Found: %s\n", servername);
 			return vhost;
 		}
@@ -391,7 +394,7 @@ lws_select_vhost(struct lws_context *context, int port, const char *servername)
 		if (port && port == vhost->listen_port &&
 		    m <= (colon - 2) &&
 		    servername[colon - m - 1] == '.' &&
-		    !strncmp(vhost->name, servername + colon - m, m)) {
+		    !strncmp(vhost->name, servername + colon - m, (unsigned int)m)) {
 			lwsl_info("SNI: Found %s on wildcard: %s\n",
 				    servername, vhost->name);
 			return vhost;
@@ -438,6 +441,7 @@ static const struct lws_mimetype {
 	{ ".txt", "text/plain" },
 	{ ".xml", "application/xml" },
 	{ ".json", "application/json" },
+	{ ".mjs", "text/javascript" },
 };
 
 const char *
@@ -592,7 +596,7 @@ lws_http_serve(struct lws *wsi, char *uri, const char *origin,
 
 #if !defined(WIN32) && !defined(LWS_PLAT_FREERTOS)
 		if ((S_IFMT & st.st_mode) == S_IFLNK) {
-			len = readlink(path, sym, sizeof(sym) - 1);
+			len = (size_t)readlink(path, sym, sizeof(sym) - 1);
 			if (len) {
 				lwsl_err("Failed to read link %s\n", path);
 				goto notfound;
@@ -677,10 +681,10 @@ lws_http_serve(struct lws *wsi, char *uri, const char *origin,
 			if (lws_finalize_http_header(wsi, &p, end))
 				return -1;
 
-			n = lws_write(wsi, start, p - start,
+			n = lws_write(wsi, start, lws_ptr_diff_size_t(p, start),
 				      LWS_WRITE_HTTP_HEADERS |
 				      LWS_WRITE_H2_STREAM_END);
-			if (n != (p - start)) {
+			if (n != lws_ptr_diff(p, start)) {
 				lwsl_err("_write returned %d from %ld\n", n,
 					 (long)(p - start));
 				return -1;
@@ -723,7 +727,7 @@ lws_http_serve(struct lws *wsi, char *uri, const char *origin,
 	while (pvo) {
 		n = (int)strlen(path);
 		if (n > (int)strlen(pvo->name) &&
-		    !strcmp(&path[n - strlen(pvo->name)], pvo->name)) {
+		    !strcmp(&path[(unsigned int)n - strlen(pvo->name)], pvo->name)) {
 			wsi->interpreting = 1;
 			if (!wsi->mux_substream)
 				wsi->sending_chunked = 1;
@@ -800,10 +804,19 @@ lws_find_mount(struct lws *wsi, const char *uri_ptr, int uri_len)
 		     uri_ptr[hm->mountpoint_len] == '/' ||
 		     hm->mountpoint_len == 1)
 		    ) {
+#if defined(LWS_WITH_SYS_METRICS)
+			lws_metrics_tag_wsi_add(wsi, "mnt", hm->mountpoint);
+#endif
+
 			if (hm->origin_protocol == LWSMPRO_CALLBACK ||
 			    ((hm->origin_protocol == LWSMPRO_CGI ||
 			     lws_hdr_total_length(wsi, WSI_TOKEN_GET_URI) ||
 			     lws_hdr_total_length(wsi, WSI_TOKEN_POST_URI) ||
+#if defined(LWS_WITH_HTTP_UNCOMMON_HEADERS)
+			     lws_hdr_total_length(wsi, WSI_TOKEN_PUT_URI) ||
+			     lws_hdr_total_length(wsi, WSI_TOKEN_PATCH_URI) ||
+			     lws_hdr_total_length(wsi, WSI_TOKEN_DELETE_URI) ||
+#endif
 			     lws_hdr_total_length(wsi, WSI_TOKEN_HEAD_URI) ||
 #if defined(LWS_ROLE_H2)
 			     (wsi->mux_substream &&
@@ -838,7 +851,7 @@ lws_find_string_in_file(const char *filename, const char *string, int stringlen)
 
 	while (1) {
 		if (pos == n) {
-			n = read(fd, buf, sizeof(buf));
+			n = (int)read(fd, buf, sizeof(buf));
 			if (n <= 0) {
 				if (match == stringlen)
 					hit = 1;
@@ -897,7 +910,7 @@ lws_unauthorised_basic_auth(struct lws *wsi)
 	if (lws_finalize_http_header(wsi, &p, end))
 		return -1;
 
-	n = lws_write(wsi, start, p - start, LWS_WRITE_HTTP_HEADERS |
+	n = lws_write(wsi, start, lws_ptr_diff_size_t(p, start), LWS_WRITE_HTTP_HEADERS |
 					     LWS_WRITE_H2_STREAM_END);
 	if (n < 0)
 		return -1;
@@ -1050,7 +1063,7 @@ lws_check_basic_auth(struct lws *wsi, const char *basic_auth_login_file,
 	case LWSAUTHM_BASIC_AUTH_CALLBACK:
 		bar = wsi->a.protocol->callback(wsi,
 				LWS_CALLBACK_VERIFY_BASIC_AUTHORIZATION,
-				wsi->user_space, plain, m);
+				wsi->user_space, plain, (unsigned int)m);
 		if (!bar)
 			return LCBA_FAILED_AUTH;
 		break;
@@ -1065,9 +1078,9 @@ lws_check_basic_auth(struct lws *wsi, const char *basic_auth_login_file,
 	 */
 
 	*pcolon = '\0';
-	wsi->http.ah->frags[fi].len = lws_ptr_diff(pcolon, plain);
+	wsi->http.ah->frags[fi].len = (uint16_t)lws_ptr_diff_size_t(pcolon, &plain[0]);
 	pcolon = lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_AUTHORIZATION);
-	strncpy(pcolon, plain, ml - 1);
+	strncpy(pcolon, plain, (unsigned int)(ml - 1));
 	pcolon[ml - 1] = '\0';
 	lwsl_info("%s: basic auth accepted for %s\n", __func__,
 		 lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_AUTHORIZATION));
@@ -1148,7 +1161,7 @@ lws_http_proxy_start(struct lws *wsi, const struct lws_http_mount *hit,
 			n = sizeof(ads) - 2;
 	}
 
-	memcpy(ads, hit->origin, n);
+	memcpy(ads, hit->origin, (unsigned int)n);
 	ads[n] = '\0';
 
 	i.address = ads;
@@ -1161,11 +1174,16 @@ lws_http_proxy_start(struct lws *wsi, const struct lws_http_mount *hit,
 		i.port = atoi(pcolon + 1);
 
 	n = lws_snprintf(rpath, sizeof(rpath) - 1, "/%s/%s",
-			 pslash + 1, uri_ptr + hit->mountpoint_len) - 2;
+			 pslash + 1, uri_ptr + hit->mountpoint_len) - 1;
 	lws_clean_url(rpath);
+	n = (int)strlen(rpath);
+	if (n && rpath[n - 1] == '/')
+		n--;
+
 	na = lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_URI_ARGS);
 	if (na) {
 		char *p;
+		int budg;
 
 		if (!n) /* don't start with the ?... use the first / if so */
 			n++;
@@ -1181,18 +1199,17 @@ lws_http_proxy_start(struct lws *wsi, const struct lws_http_mount *hit,
 		}
 
 		*p++ = '?';
-		if (lws_hdr_copy(wsi, p,
+		budg = lws_hdr_copy(wsi, p,
 			     (int)(&rpath[sizeof(rpath) - 1] - p),
-			     WSI_TOKEN_HTTP_URI_ARGS) > 0)
-			while (na--) {
-				if (*p == '\0')
-					*p = '&';
-				p++;
-			}
+			     WSI_TOKEN_HTTP_URI_ARGS);
+	       if (budg > 0)
+		       p += budg;
+
 		*p = '\0';
 	}
 
 	i.path = rpath;
+	lwsl_notice("%s: proxied path '%s'\n", __func__, i.path);
 
 	/* incoming may be h1 or h2... if he sends h1 HOST, use that
 	 * directly, otherwise we must convert h2 :authority to h1
@@ -1277,8 +1294,8 @@ lws_http_proxy_start(struct lws *wsi, const struct lws_http_mount *hit,
 		return 1;
 	}
 
-	lwsl_info("%s: setting proxy clientside on %p (parent %p)\n",
-		  __func__, cwsi, lws_get_parent(cwsi));
+	lwsl_info("%s: setting proxy clientside on %s (parent %s)\n",
+		  __func__, lws_wsi_tag(cwsi), lws_wsi_tag(lws_get_parent(cwsi)));
 
 	cwsi->http.proxy_clientside = 1;
 	if (ws) {
@@ -1410,6 +1427,9 @@ lws_http_action(struct lws *wsi)
 	if (meth < 0 || meth >= (int)LWS_ARRAY_SIZE(method_names))
 		goto bail_nuke_ah;
 
+	lws_metrics_tag_wsi_add(wsi, "vh", wsi->a.vhost->name);
+	lws_metrics_tag_wsi_add(wsi, "meth", method_names[meth]);
+
 	/* we insist on absolute paths */
 
 	if (!uri_ptr || uri_ptr[0] != '/') {
@@ -1421,8 +1441,11 @@ lws_http_action(struct lws *wsi)
 	lwsl_info("Method: '%s' (%d), request for '%s'\n", method_names[meth],
 		  meth, uri_ptr);
 
-	if (wsi->role_ops && wsi->role_ops->check_upgrades)
-		switch (wsi->role_ops->check_upgrades(wsi)) {
+	if (wsi->role_ops &&
+	    lws_rops_fidx(wsi->role_ops, LWS_ROPS_check_upgrades))
+		switch (lws_rops_func_fidx(wsi->role_ops,
+					   LWS_ROPS_check_upgrades).
+							check_upgrades(wsi)) {
 		case LWS_UPG_RET_DONE:
 			return 0;
 		case LWS_UPG_RET_CONTINUE:
@@ -1452,7 +1475,7 @@ lws_http_action(struct lws *wsi)
 			 sizeof(content_length_str) - 1,
 			 WSI_TOKEN_HTTP_CONTENT_LENGTH) > 0) {
 		wsi->http.rx_content_remain = wsi->http.rx_content_length =
-						atoll(content_length_str);
+				(lws_filepos_t)atoll(content_length_str);
 		if (!wsi->http.rx_content_length) {
 			wsi->http.content_length_explicitly_zero = 1;
 			lwsl_debug("%s: explicit 0 content-length\n", __func__);
@@ -1496,8 +1519,8 @@ lws_http_action(struct lws *wsi)
 		wsi->http.conn_type = conn_type;
 	}
 
-	n = wsi->a.protocol->callback(wsi, LWS_CALLBACK_FILTER_HTTP_CONNECTION,
-				    wsi->user_space, uri_ptr, uri_len);
+	n = (unsigned int)wsi->a.protocol->callback(wsi, LWS_CALLBACK_FILTER_HTTP_CONNECTION,
+				    wsi->user_space, uri_ptr, (unsigned int)uri_len);
 	if (n) {
 		lwsl_info("LWS_CALLBACK_HTTP closing\n");
 
@@ -1509,34 +1532,54 @@ lws_http_action(struct lws *wsi)
 	 */
 	if (!wsi->mux_stream_immortal)
 		lws_set_timeout(wsi, PENDING_TIMEOUT_HTTP_CONTENT,
-				wsi->a.context->timeout_secs);
-#ifdef LWS_WITH_TLS
+				(int)wsi->a.context->timeout_secs);
+#if defined(LWS_WITH_TLS)
 	if (wsi->tls.redirect_to_https) {
 		/*
-		 * we accepted http:// only so we could redirect to
+		 * We accepted http:// only so we could redirect to
 		 * https://, so issue the redirect.  Create the redirection
-		 * URI from the host: header and ignore the path part
+		 * URI from the host: header, and regenerate the path part from
+		 * the parsed pieces
 		 */
 		unsigned char *start = pt->serv_buf + LWS_PRE, *p = start,
-			      *end = p + wsi->a.context->pt_serv_buf_size - LWS_PRE;
+			      *end = p + wsi->a.context->pt_serv_buf_size -
+				     LWS_PRE;
 
-		n = lws_hdr_total_length(wsi, WSI_TOKEN_HOST);
+		n = (unsigned int)lws_hdr_total_length(wsi, WSI_TOKEN_HOST);
 		if (!n || n > 128)
 			goto bail_nuke_ah;
 
 		if (!lws_hdr_simple_ptr(wsi, WSI_TOKEN_HOST))
 			goto bail_nuke_ah;
 
-		p += lws_snprintf((char *)p, lws_ptr_diff(end, p), "https://");
+		p += lws_snprintf((char *)p, lws_ptr_diff_size_t(end, p), "https://");
 		memcpy(p, lws_hdr_simple_ptr(wsi, WSI_TOKEN_HOST), n);
 		p += n;
 		*p++ = '/';
-		*p = '\0';
-		n = lws_ptr_diff(p, start);
+		if (uri_len >= lws_ptr_diff(end, p))
+			goto bail_nuke_ah;
+
+		if (uri_ptr[0])
+			p--;
+		memcpy(p, uri_ptr, (unsigned int)uri_len);
+		p += uri_len;
+
+		n = 0;
+		while (lws_hdr_copy_fragment(wsi, (char *)p + 1,
+					     lws_ptr_diff(end, p) - 2,
+					     WSI_TOKEN_HTTP_URI_ARGS, (int)n) > 0) {
+			*p = n ? '&' : '?';
+			p += strlen((char *)p);
+			if (p >= end - 2)
+				goto bail_nuke_ah;
+			n++;
+		}
+
+		n = (unsigned int)lws_ptr_diff(p, start);
 
 		p += LWS_PRE;
-		n = lws_http_redirect(wsi, HTTP_STATUS_MOVED_PERMANENTLY,
-				      start, n, &p, end);
+		n = (unsigned int)lws_http_redirect(wsi, HTTP_STATUS_MOVED_PERMANENTLY,
+				      start, (int)n, &p, end);
 		if ((int)n < 0)
 			goto bail_nuke_ah;
 
@@ -1563,7 +1606,7 @@ lws_http_action(struct lws *wsi)
 		lwsi_set_state(wsi, LRS_DOING_TRANSACTION);
 
 		m = wsi->a.protocol->callback(wsi, LWS_CALLBACK_HTTP,
-				    wsi->user_space, uri_ptr, uri_len);
+				    wsi->user_space, uri_ptr, (unsigned int)uri_len);
 
 		goto after;
 	}
@@ -1571,9 +1614,9 @@ lws_http_action(struct lws *wsi)
 #if defined(LWS_WITH_FILE_OPS)
 	s = uri_ptr + hit->mountpoint_len;
 #endif
-	n = lws_http_redirect_hit(pt, wsi, hit, uri_ptr, uri_len, &ha);
+	n = (unsigned int)lws_http_redirect_hit(pt, wsi, hit, uri_ptr, uri_len, &ha);
 	if (ha)
-		return n;
+		return (int)n;
 
 #if defined(LWS_WITH_HTTP_BASIC_AUTH)
 
@@ -1603,10 +1646,10 @@ lws_http_action(struct lws *wsi)
 
 	if (hit->origin_protocol == LWSMPRO_HTTPS ||
 	    hit->origin_protocol == LWSMPRO_HTTP) {
-		n = lws_http_proxy_start(wsi, hit, uri_ptr, 0);
+		n = (unsigned int)lws_http_proxy_start(wsi, hit, uri_ptr, 0);
 		// lwsl_notice("proxy start says %d\n", n);
 		if (n)
-			return n;
+			return (int)n;
 
 		goto deal_body;
 	}
@@ -1643,7 +1686,7 @@ lws_http_action(struct lws *wsi)
 		args.final = 0; /* used to signal callback dealt with it */
 		args.chunked = 0;
 
-		n = wsi->a.protocol->callback(wsi,
+		n = (unsigned int)wsi->a.protocol->callback(wsi,
 					    LWS_CALLBACK_CHECK_ACCESS_RIGHTS,
 					    wsi->user_space, &args, 0);
 		if (n) {
@@ -1663,7 +1706,7 @@ lws_http_action(struct lws *wsi)
 			m = wsi->a.protocol->callback(wsi, LWS_CALLBACK_HTTP,
 					    wsi->user_space,
 					    uri_ptr + hit->mountpoint_len,
-					    uri_len - hit->mountpoint_len);
+					    (unsigned int)uri_len - hit->mountpoint_len);
 			goto after;
 		}
 	}
@@ -1681,9 +1724,9 @@ lws_http_action(struct lws *wsi)
 
 		n = 5;
 		if (hit->cgi_timeout)
-			n = hit->cgi_timeout;
+			n = (unsigned int)hit->cgi_timeout;
 
-		n = lws_cgi(wsi, cmd, hit->mountpoint_len, n,
+		n = (unsigned int)lws_cgi(wsi, cmd, hit->mountpoint_len, (int)n,
 			    hit->cgienv);
 		if (n) {
 			lwsl_err("%s: cgi failed\n", __func__);
@@ -1695,14 +1738,14 @@ lws_http_action(struct lws *wsi)
 #endif
 
 #if defined(LWS_WITH_FILE_OPS)
-	n = uri_len - lws_ptr_diff(s, uri_ptr);
+	n = (unsigned int)(uri_len - lws_ptr_diff(s, uri_ptr));
 	if (s[0] == '\0' || (n == 1 && s[n - 1] == '/'))
 		s = (char *)hit->def;
 	if (!s)
 		s = "index.html";
 #endif
 
-	wsi->cache_secs = hit->cache_max_age;
+	wsi->cache_secs = (unsigned int)hit->cache_max_age;
 	wsi->cache_reuse = hit->cache_reusable;
 	wsi->cache_revalidate = hit->cache_revalidate;
 	wsi->cache_intermediaries = hit->cache_intermediaries;
@@ -1735,10 +1778,10 @@ lws_http_action(struct lws *wsi)
 			m = pp->callback(wsi, LWS_CALLBACK_HTTP,
 					 wsi->user_space,
 					 uri_ptr + hit->mountpoint_len,
-					 uri_len - hit->mountpoint_len);
+					 (size_t)(uri_len - hit->mountpoint_len));
 		} else
 			m = wsi->a.protocol->callback(wsi, LWS_CALLBACK_HTTP,
-				    wsi->user_space, uri_ptr, uri_len);
+				    wsi->user_space, uri_ptr, (size_t)uri_len);
 	}
 
 after:
@@ -1796,8 +1839,8 @@ deal_body:
 
 	if (lwsi_state(wsi) != LRS_DISCARD_BODY) {
 		lwsi_set_state(wsi, LRS_BODY);
-		lwsl_info("%s: %p: LRS_BODY state set (0x%x)\n", __func__, wsi,
-			  (int)wsi->wsistate);
+		lwsl_info("%s: %s: LRS_BODY state set (0x%x)\n", __func__,
+			  lws_wsi_tag(wsi), (int)wsi->wsistate);
 	}
 	wsi->http.rx_content_remain = wsi->http.rx_content_length;
 
@@ -1818,7 +1861,7 @@ deal_body:
 			break;
 
 		lwsl_debug("%s: consuming %d\n", __func__, (int)ebuf.len);
-		m = lws_read_h1(wsi, ebuf.token, ebuf.len);
+		m = lws_read_h1(wsi, ebuf.token, (lws_filepos_t)ebuf.len);
 		if (m < 0)
 			return -1;
 
@@ -1867,13 +1910,13 @@ lws_confirm_host_header(struct lws *wsi)
 		lwsl_info("%s: missing or oversize host header\n", __func__);
 		return 1;
 	}
-	ts.len = n;
+	ts.len = (size_t)n;
 
 	if (lws_tokenize(&ts) != LWS_TOKZE_TOKEN)
 		goto bad_format;
 
 	if (strncmp(ts.token, wsi->a.vhost->name, ts.token_len)) {
-		buf[(ts.token - buf) + ts.token_len] = '\0';
+		buf[(size_t)(ts.token - buf) + ts.token_len] = '\0';
 		lwsl_info("%s: '%s' in host hdr but vhost name %s\n",
 			  __func__, ts.token, wsi->a.vhost->name);
 		return 1;
@@ -1947,13 +1990,13 @@ lws_http_to_fallback(struct lws *wsi, unsigned char *obuf, size_t olen)
 		    ipbuf, role ? role->name : "null", protocol->name, n,
 		    wsi->http.ah);
 
-	if ((wsi->a.protocol->callback)(wsi, n, wsi->user_space, NULL, 0))
+	if ((wsi->a.protocol->callback)(wsi, (enum lws_callback_reasons)n, wsi->user_space, NULL, 0))
 		return 1;
 
 	n = LWS_CALLBACK_RAW_RX;
 	if (wsi->role_ops->rx_cb[lwsi_role_server(wsi)])
 		n = wsi->role_ops->rx_cb[lwsi_role_server(wsi)];
-	if (wsi->a.protocol->callback(wsi, n, wsi->user_space, obuf, olen))
+	if (wsi->a.protocol->callback(wsi, (enum lws_callback_reasons)n, wsi->user_space, obuf, olen))
 		return 1;
 
 	return 0;
@@ -1995,7 +2038,7 @@ lws_handshake_server(struct lws *wsi, unsigned char **buf, size_t len)
 		m = lws_parse(wsi, *buf, &i);
 		lwsl_info("%s: parsed count %d\n", __func__, (int)len - i);
 		(*buf) += (int)len - i;
-		len = i;
+		len = (unsigned int)i;
 
 		if (m == LPR_DO_FALLBACK) {
 
@@ -2047,17 +2090,9 @@ raw_transition:
 		} else
 			lwsl_info("no host\n");
 
-		if (!lwsi_role_h2(wsi) || !lwsi_role_server(wsi)) {
-#if defined(LWS_WITH_SERVER_STATUS)
-			wsi->a.vhost->conn_stats.h1_trans++;
-#endif
-			if (!wsi->conn_stat_done) {
-#if defined(LWS_WITH_SERVER_STATUS)
-				wsi->a.vhost->conn_stats.h1_conn++;
-#endif
-				wsi->conn_stat_done = 1;
-			}
-		}
+		if ((!lwsi_role_h2(wsi) || !lwsi_role_server(wsi)) &&
+		    (!wsi->conn_stat_done))
+			wsi->conn_stat_done = 1;
 
 		/* check for unwelcome guests */
 #if defined(LWS_WITH_HTTP_UNCOMMON_HEADERS)
@@ -2083,7 +2118,7 @@ raw_transition:
 					if (msg)
 						msg++;
 					lws_return_http_status(wsi,
-						atoi(rej->value), msg);
+						(unsigned int)atoi(rej->value), msg);
 #ifdef LWS_WITH_ACCESS_LOG
 					meth = lws_http_get_uri_and_method(wsi,
 							&uri_ptr, &uri_len);
@@ -2092,9 +2127,6 @@ raw_transition:
 							uri_ptr, uri_len, meth);
 
 					/* wsi close will do the log */
-#endif
-#if defined(LWS_WITH_SERVER_STATUS)
-					wsi->a.vhost->conn_stats.rejected++;
 #endif
 					/*
 					 * We don't want anything from
@@ -2186,18 +2218,14 @@ raw_transition:
 
 			if (!strcasecmp(up, "websocket")) {
 #if defined(LWS_ROLE_WS)
-#if defined(LWS_WITH_SERVER_STATUS)
-				wsi->a.vhost->conn_stats.ws_upg++;
-#endif
+				lws_metrics_tag_wsi_add(wsi, "upg", "ws");
 				lwsl_info("Upgrade to ws\n");
 				goto upgrade_ws;
 #endif
 			}
 #if defined(LWS_WITH_HTTP2)
 			if (!strcasecmp(up, "h2c")) {
-#if defined(LWS_WITH_SERVER_STATUS)
-				wsi->a.vhost->conn_stats.h2_upg++;
-#endif
+				lws_metrics_tag_wsi_add(wsi, "upg", "h2c");
 				lwsl_info("Upgrade to h2c\n");
 				goto upgrade_h2c;
 			}
@@ -2206,7 +2234,7 @@ raw_transition:
 
 		/* no upgrade ack... he remained as HTTP */
 
-		lwsl_info("%s: %p: No upgrade\n", __func__, wsi);
+		lwsl_info("%s: %s: No upgrade\n", __func__, lws_wsi_tag(wsi));
 
 		lwsi_set_state(wsi, LRS_ESTABLISHED);
 #if defined(LWS_WITH_FILE_OPS)
@@ -2217,7 +2245,7 @@ raw_transition:
 		lws_http_compression_validate(wsi);
 #endif
 
-		lwsl_debug("%s: wsi %p: ah %p\n", __func__, (void *)wsi,
+		lwsl_debug("%s: %s: ah %p\n", __func__, lws_wsi_tag(wsi),
 			   (void *)wsi->http.ah);
 
 		n = lws_http_action(wsi);
@@ -2264,14 +2292,14 @@ upgrade_h2c:
 
 		lws_h2_settings(wsi, &wsi->h2.h2n->peer_set, (uint8_t *)tbuf, n);
 
-		lws_hpack_dynamic_size(wsi, wsi->h2.h2n->peer_set.s[
+		lws_hpack_dynamic_size(wsi, (int)wsi->h2.h2n->peer_set.s[
 		                                      H2SET_HEADER_TABLE_SIZE]);
 
 		strcpy(tbuf, "HTTP/1.1 101 Switching Protocols\x0d\x0a"
 			      "Connection: Upgrade\x0d\x0a"
 			      "Upgrade: h2c\x0d\x0a\x0d\x0a");
 		m = (int)strlen(tbuf);
-		n = lws_issue_raw(wsi, (unsigned char *)tbuf, m);
+		n = lws_issue_raw(wsi, (unsigned char *)tbuf, (unsigned int)m);
 		if (n != m) {
 			lwsl_debug("http2 switch: ERROR writing to socket\n");
 			return 1;
@@ -2320,7 +2348,8 @@ lws_http_transaction_completed(struct lws *wsi)
 		 * Defer the transaction completed until the last part of the
 		 * partial is sent.
 		 */
-		lwsl_debug("%s: %p: deferring due to partial\n", __func__, wsi);
+		lwsl_debug("%s: %s: deferring due to partial\n", __func__,
+				lws_wsi_tag(wsi));
 		wsi->http.deferred_transaction_completed = 1;
 		lws_callback_on_writable(wsi);
 
@@ -2349,7 +2378,16 @@ lws_http_transaction_completed(struct lws *wsi)
 		return 0;
 	}
 
-	lwsl_info("%s: wsi %p\n", __func__, wsi);
+#if defined(LWS_WITH_SYS_METRICS)
+	{
+		char tmp[10];
+
+		lws_snprintf(tmp, sizeof(tmp), "%u", wsi->http.response_code);
+		lws_metrics_tag_wsi_add(wsi, "status", tmp);
+	}
+#endif
+
+	lwsl_info("%s: %s\n", __func__, lws_wsi_tag(wsi));
 
 #if defined(LWS_WITH_HTTP_STREAM_COMPRESSION)
 	lws_http_compression_destroy(wsi);
@@ -2395,7 +2433,7 @@ lws_http_transaction_completed(struct lws *wsi)
 		return 1;
 
 	if (wsi->http.conn_type != HTTP_CONNECTION_KEEP_ALIVE) {
-		lwsl_info("%s: %p: close connection\n", __func__, wsi);
+		lwsl_info("%s: %s: close connection\n", __func__, lws_wsi_tag(wsi));
 		return 1;
 	}
 
@@ -2408,8 +2446,8 @@ lws_http_transaction_completed(struct lws *wsi)
 	 * until we can verify POLLOUT.  The part of this that confirms POLLOUT
 	 * with no partials is in lws_server_socket_service() below.
 	 */
-	lwsl_debug("%s: %p: setting DEF_ACT from 0x%x: %p\n", __func__,
-		   wsi, (int)wsi->wsistate, wsi->buflist);
+	lwsl_debug("%s: %s: setting DEF_ACT from 0x%x: %p\n", __func__,
+		   lws_wsi_tag(wsi), (int)wsi->wsistate, wsi->buflist);
 	lwsi_set_state(wsi, LRS_DEFERRING_ACTION);
 	wsi->http.tx_content_length = 0;
 	wsi->http.tx_content_remain = 0;
@@ -2427,7 +2465,7 @@ lws_http_transaction_completed(struct lws *wsi)
 	n = NO_PENDING_TIMEOUT;
 	if (wsi->a.vhost->keepalive_timeout)
 		n = PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE;
-	lws_set_timeout(wsi, n, wsi->a.vhost->keepalive_timeout);
+	lws_set_timeout(wsi, (enum pending_timeout)n, wsi->a.vhost->keepalive_timeout);
 
 	/*
 	 * We already know we are on http1.1 / keepalive and the next thing
@@ -2444,8 +2482,8 @@ lws_http_transaction_completed(struct lws *wsi)
 	if (wsi->http.ah) {
 		// lws_buflist_describe(&wsi->buflist, wsi, __func__);
 		if (!lws_buflist_next_segment_len(&wsi->buflist, NULL)) {
-			lwsl_debug("%s: %p: nothing in buflist, detaching ah\n",
-				  __func__, wsi);
+			lwsl_debug("%s: %s: nothing in buflist, detaching ah\n",
+				  __func__, lws_wsi_tag(wsi));
 			lws_header_table_detach(wsi, 1);
 #ifdef LWS_WITH_TLS
 			/*
@@ -2464,8 +2502,8 @@ lws_http_transaction_completed(struct lws *wsi)
 			}
 #endif
 		} else {
-			lwsl_info("%s: %p: resetting/keeping ah as pipeline\n",
-				  __func__, wsi);
+			lwsl_info("%s: %s: resetting/keeping ah as pipeline\n",
+				  __func__, lws_wsi_tag(wsi));
 			lws_header_table_reset(wsi, 0);
 			/*
 			 * If we kept the ah, we should restrict the amount
@@ -2486,8 +2524,8 @@ lws_http_transaction_completed(struct lws *wsi)
 			if (lws_header_table_attach(wsi, 0))
 				lwsl_debug("acquired ah\n");
 
-	lwsl_debug("%s: %p: keep-alive await new transaction (state 0x%x)\n",
-		   __func__, wsi, (int)wsi->wsistate);
+	lwsl_debug("%s: %s: keep-alive await new transaction (state 0x%x)\n",
+		   __func__, lws_wsi_tag(wsi), (int)wsi->wsistate);
 	lws_callback_on_writable(wsi);
 
 	return 0;
@@ -2574,7 +2612,7 @@ lws_serve_http_file(struct lws *wsi, const char *file, const char *content_type,
 		n = HTTP_STATUS_PARTIAL_CONTENT;
 #endif
 
-	if (lws_add_http_header_status(wsi, n, &p, end))
+	if (lws_add_http_header_status(wsi, (unsigned int)n, &p, end))
 		goto bail;
 
 	if ((wsi->http.fop_fd->flags & (LWS_FOP_FLAG_COMPR_ACCEPTABLE_GZIP |
@@ -2648,13 +2686,14 @@ lws_serve_http_file(struct lws *wsi, const char *file, const char *content_type,
 					"bytes %llu-%llu/%llu",
 					rp->start, rp->end, rp->extent);
 
-			total_content_length +=
+			total_content_length = total_content_length +
+					(lws_filepos_t)(
 				6 /* header _lws\r\n */ +
 				/* Content-Type: xxx/xxx\r\n */
-				14 + strlen(content_type) + 2 +
+				14 + (int)strlen(content_type) + 2 +
 				/* Content-Range: xxxx\r\n */
 				15 + n + 2 +
-				2; /* /r/n */
+				2); /* /r/n */
 		}
 
 		lws_ranges_reset(rp);
@@ -2758,14 +2797,14 @@ lws_serve_http_file(struct lws *wsi, const char *file, const char *content_type,
 	if (other_headers) {
 		if ((end - p) < other_headers_len)
 			goto bail;
-		memcpy(p, other_headers, other_headers_len);
+		memcpy(p, other_headers, (unsigned int)other_headers_len);
 		p += other_headers_len;
 	}
 
 	if (lws_finalize_http_header(wsi, &p, end))
 		goto bail;
 
-	ret = lws_write(wsi, response, p - response, LWS_WRITE_HTTP_HEADERS);
+	ret = lws_write(wsi, response, lws_ptr_diff_size_t(p, response), LWS_WRITE_HTTP_HEADERS);
 	if (ret != (p - response)) {
 		lwsl_err("_write returned %d from %ld\n", ret,
 			 (long)(p - response));
@@ -2834,7 +2873,9 @@ int lws_serve_http_file_fragment(struct lws *wsi)
 			   __func__, wsi->http.comp_ctx.buflist_comp,
 			   wsi->http.comp_ctx.may_have_more);
 
-		if (wsi->role_ops->write_role_protocol(wsi, NULL, 0, &wp) < 0) {
+		if (lws_rops_fidx(wsi->role_ops, LWS_ROPS_write_role_protocol) &&
+		    lws_rops_func_fidx(wsi->role_ops, LWS_ROPS_write_role_protocol).
+					write_role_protocol(wsi, NULL, 0, &wp) < 0) {
 			lwsl_info("%s signalling to close\n", __func__);
 			goto file_had_it;
 		}
@@ -2857,8 +2898,8 @@ int lws_serve_http_file_fragment(struct lws *wsi)
 				    wsi->http.range.start);
 
 			if ((long long)lws_vfs_file_seek_cur(wsi->http.fop_fd,
-						   wsi->http.range.start -
-						   wsi->http.filepos) < 0)
+						   (lws_fileofs_t)wsi->http.range.start -
+						   (lws_fileofs_t)wsi->http.filepos) < 0)
 				goto file_had_it;
 
 			wsi->http.filepos = wsi->http.range.start;
@@ -2885,7 +2926,7 @@ int lws_serve_http_file_fragment(struct lws *wsi)
 		}
 #endif
 
-		poss = context->pt_serv_buf_size - n -
+		poss = context->pt_serv_buf_size - (unsigned int)n -
 				LWS_H2_FRAME_HEADER_LENGTH;
 
 		if (wsi->http.tx_content_length)
@@ -2900,17 +2941,18 @@ int lws_serve_http_file_fragment(struct lws *wsi)
 		    poss > wsi->a.protocol->tx_packet_size)
 			poss = wsi->a.protocol->tx_packet_size;
 
-		if (wsi->role_ops->tx_credit) {
-			lws_filepos_t txc =
-				wsi->role_ops->tx_credit(wsi, LWSTXCR_US_TO_PEER, 0);
+		if (lws_rops_fidx(wsi->role_ops, LWS_ROPS_tx_credit)) {
+			lws_filepos_t txc = (unsigned int)lws_rops_func_fidx(wsi->role_ops,
+							       LWS_ROPS_tx_credit).
+					tx_credit(wsi, LWSTXCR_US_TO_PEER, 0);
 
 			if (!txc) {
 				/*
 				 * We shouldn't've been able to get the
 				 * WRITEABLE if we are skint
 				 */
-				lwsl_notice("%s: %p: no tx credit\n", __func__,
-						wsi);
+				lwsl_notice("%s: %s: no tx credit\n", __func__,
+						lws_wsi_tag(wsi));
 
 				return 0;
 			}
@@ -2951,14 +2993,14 @@ int lws_serve_http_file_fragment(struct lws *wsi)
 
 		if (n) {
 			lws_set_timeout(wsi, PENDING_TIMEOUT_HTTP_CONTENT,
-					context->timeout_secs);
+					(int)context->timeout_secs);
 
 			if (wsi->interpreting) {
 				args.p = (char *)p;
 				args.len = n;
-				args.max_len = (unsigned int)poss + 128;
-				args.final = wsi->http.filepos + n ==
-					     wsi->http.filelen;
+				args.max_len = (int)(unsigned int)poss + 128;
+				args.final = wsi->http.filepos + (unsigned int)n ==
+							wsi->http.filelen;
 				args.chunked = wsi->sending_chunked;
 				if (user_callback_handle_rxflow(
 				     wsi->a.vhost->protocols[
@@ -2981,7 +3023,7 @@ int lws_serve_http_file_fragment(struct lws *wsi)
 				lwsl_debug("added trailing boundary\n");
 			}
 #endif
-			m = lws_write(wsi, p, n, wsi->http.filepos + amount ==
+			m = lws_write(wsi, p, (unsigned int)n, wsi->http.filepos + amount ==
 					wsi->http.filelen ?
 					 LWS_WRITE_HTTP_FINAL : LWS_WRITE_HTTP);
 			if (m < 0)
@@ -3129,14 +3171,14 @@ lws_chunked_html_process(struct lws_process_html_args *args,
 			if (s->pos == sizeof(s->swallow) - 1)
 				goto skip;
 			for (n = 0; n < s->count_vars; n++)
-				if (!strncmp(s->swallow, s->vars[n], s->pos)) {
+				if (!strncmp(s->swallow, s->vars[n], (unsigned int)s->pos)) {
 					hits++;
 					hit = n;
 				}
 			if (!hits) {
 skip:
 				s->swallow[s->pos] = '\0';
-				memcpy(s->start, s->swallow, s->pos);
+				memcpy(s->start, s->swallow, (unsigned int)s->pos);
 				args->len++;
 				s->pos = 0;
 				sp = s->start + 1;
@@ -3150,10 +3192,10 @@ skip:
 				s->swallow[s->pos] = '\0';
 				if (n != s->pos) {
 					memmove(s->start + n, s->start + s->pos,
-						old_len - (sp - args->p) - 1);
+						(unsigned int)(old_len - (sp - args->p) - 1));
 					old_len += (n - s->pos) + 1;
 				}
-				memcpy(s->start, pc, n);
+				memcpy(s->start, pc, (unsigned int)n);
 				args->len++;
 				sp = s->start + 1;
 
@@ -3175,7 +3217,7 @@ skip:
 		n = sprintf(buffer, "%X\x0d\x0a", args->len);
 
 		args->p -= n;
-		memcpy(args->p, buffer, n);
+		memcpy(args->p, buffer, (unsigned int)n);
 		args->len += n;
 
 		if (args->final) {
