@@ -1,7 +1,7 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2019 - 2020 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2019 - 2021 Andy Green <andy@warmcat.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -43,6 +43,9 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 			 in ? (char *)in : "(null)");
 		if (!h)
 			break;
+
+		lws_conmon_ss_json(h);
+
 		r = lws_ss_event_helper(h, LWSSSCS_UNREACHABLE);
 		h->wsi = NULL;
 
@@ -52,11 +55,11 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		}
 
 		if (r == LWSSSSRET_DESTROY_ME)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		r = lws_ss_backoff(h);
 		if (r != LWSSSSRET_OK)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		break;
 
@@ -64,7 +67,13 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		if (!h)
 			break;
 		lws_sul_cancel(&h->sul_timeout);
-		r= lws_ss_event_helper(h, LWSSSCS_DISCONNECTED);
+
+		lws_conmon_ss_json(h);
+
+		if (h->ss_dangling_connected)
+			r = lws_ss_event_helper(h, LWSSSCS_DISCONNECTED);
+		else
+			r = lws_ss_event_helper(h, LWSSSCS_UNREACHABLE);
 		if (h->wsi)
 			lws_set_opaque_user_data(h->wsi, NULL);
 		h->wsi = NULL;
@@ -75,13 +84,13 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		}
 
 		if (r)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		if (h->policy && !(h->policy->flags & LWSSSPOLF_OPPORTUNISTIC) &&
 		    !h->txn_ok && !wsi->a.context->being_destroyed) {
 			r = lws_ss_backoff(h);
 			if (r != LWSSSSRET_OK)
-				return _lws_ss_handle_state_ret(r, wsi, &h);
+				return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 		}
 		break;
 
@@ -93,10 +102,26 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		h->wsi = wsi;
 		h->retry = 0;
 		h->seqstate = SSSEQ_CONNECTED;
+		/*
+		 * If a subscribe is pending on the stream, then make
+		 * sure the SUBSCRIBE is done before signaling the
+		 * user application.
+		 */
+		if (h->policy->u.mqtt.subscribe &&
+		    !wsi->mqtt->done_subscribe) {
+			lws_callback_on_writable(wsi);
+			break;
+		}
 		lws_sul_cancel(&h->sul);
+#if defined(LWS_WITH_SYS_METRICS)
+		/*
+		 * If any hanging caliper measurement, dump it, and free any tags
+		 */
+		lws_metrics_caliper_report_hist(h->cal_txn, (struct lws *)NULL);
+#endif
 		r = lws_ss_event_helper(h, LWSSSCS_CONNECTED);
 		if (r != LWSSSSRET_OK)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 		if (h->policy->u.mqtt.topic)
 			lws_callback_on_writable(wsi);
 		break;
@@ -119,11 +144,22 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		r = h->info.rx(ss_to_userobj(h), (const uint8_t *)pmqpp->payload,
 			   len, f);
 		if (r != LWSSSSRET_OK)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		return 0; /* don't passthru */
 
 	case LWS_CALLBACK_MQTT_SUBSCRIBED:
+		/*
+		 * Stream demanded a subscribe while connecting, once
+		 * done notify CONNECTED event to the application.
+		 */
+		if (wsi->mqtt->done_subscribe == 0) {
+			lws_sul_cancel(&h->sul);
+			r = lws_ss_event_helper(h, LWSSSCS_CONNECTED);
+			if (r != LWSSSSRET_OK)
+				return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r,
+									      wsi, &h);
+		}
 		wsi->mqtt->done_subscribe = 1;
 		lws_callback_on_writable(wsi);
 		break;
@@ -132,21 +168,65 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		lws_sul_cancel(&h->sul_timeout);
 		r = lws_ss_event_helper(h, LWSSSCS_QOS_ACK_REMOTE);
 		if (r != LWSSSSRET_OK)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 		break;
 
 	case LWS_CALLBACK_MQTT_CLIENT_WRITEABLE:
+	{
+		size_t used_in, used_out, topic_limit;
+		lws_strexp_t exp;
+		char *expbuf;
+
 		if (!h || !h->info.tx)
 			return 0;
-		lwsl_notice("%s: ss %p: WRITEABLE\n", __func__, h);
+		lwsl_notice("%s: %s: WRITEABLE\n", __func__, lws_ss_tag(h));
 
 		if (h->seqstate != SSSEQ_CONNECTED) {
 			lwsl_warn("%s: seqstate %d\n", __func__, h->seqstate);
 			break;
 		}
+		if (h->policy->u.mqtt.aws_iot)
+			topic_limit = LWS_MQTT_MAX_AWSIOT_TOPICLEN;
+		else
+			topic_limit = LWS_MQTT_MAX_TOPICLEN;
 
 		if (h->policy->u.mqtt.subscribe &&
 		    !wsi->mqtt->done_subscribe) {
+			lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata,
+					NULL, topic_limit);
+			/*
+			 * Expand with no output first to calculate the size of
+			 * expanded string then, allocate new buffer and expand
+			 * again with the buffer
+			 */
+			if (lws_strexp_expand(&exp, h->policy->u.mqtt.subscribe,
+					      strlen(h->policy->u.mqtt.subscribe),
+					      &used_in, &used_out) != LSTRX_DONE) {
+				lwsl_err("%s, failed to expand MQTT subscribe"
+					 " topic with no output\n", __func__);
+				return 1;
+			}
+
+			expbuf = lws_malloc(used_out + 1, __func__);
+			if (!expbuf) {
+				lwsl_err("%s, failed to allocate MQTT subscribe"
+				         "topic", __func__);
+				return 1;
+			}
+
+			lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata,
+					expbuf, used_out);
+
+			if (lws_strexp_expand(&exp, h->policy->u.mqtt.subscribe,
+					      strlen(h->policy->u.mqtt.subscribe),
+					      &used_in, &used_out) != LSTRX_DONE) {
+				lwsl_err("%s, failed to expand MQTT subscribe topic\n",
+					 __func__);
+				lws_free(expbuf);
+				return 1;
+			}
+			lwsl_notice("%s, expbuf - %s\n", __func__, expbuf);
+			h->u.mqtt.sub_top.name = expbuf;
 
 			/*
 			 * The policy says to subscribe to something, and we
@@ -155,10 +235,8 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 			 */
 
 			lwsl_notice("%s: subscribing %s\n", __func__,
-						h->u.mqtt.subscribe_to);
+				                h->u.mqtt.sub_top.name);
 
-			memset(&h->u.mqtt.sub_top, 0, sizeof(h->u.mqtt.sub_top));
-			h->u.mqtt.sub_top.name = h->u.mqtt.subscribe_to;
 			h->u.mqtt.sub_top.qos = h->policy->u.mqtt.qos;
 			memset(&h->u.mqtt.sub_info, 0, sizeof(h->u.mqtt.sub_info));
 			h->u.mqtt.sub_info.num_topics = 1;
@@ -166,6 +244,15 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 
 			if (lws_mqtt_client_send_subcribe(wsi, &h->u.mqtt.sub_info)) {
 				lwsl_notice("%s: unable to subscribe", __func__);
+				lws_free(expbuf);
+				h->u.mqtt.sub_top.name = NULL;
+				return -1;
+			}
+			lws_free(expbuf);
+			h->u.mqtt.sub_top.name = NULL;
+			/* Expect a SUBACK */
+			if (lws_change_pollfd(wsi, 0, LWS_POLLIN)) {
+				lwsl_err("%s: Unable to set LWS_POLLIN\n", __func__);
 				return -1;
 			}
 
@@ -180,18 +267,47 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 			return 0;
 
 		if (r < 0)
-			return _lws_ss_handle_state_ret(r, wsi, &h);
+			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
 		memset(&mqpp, 0, sizeof(mqpp));
 		/* this is the string-substituted h->policy->u.mqtt.topic */
 		mqpp.topic = (char *)h->u.mqtt.topic_qos.name;
-		mqpp.topic_len = strlen(mqpp.topic);
-		mqpp.packet_id = h->txord - 1;
+		lws_strexp_init(&exp, h, lws_ss_exp_cb_metadata, NULL,
+				topic_limit);
+
+		if (lws_strexp_expand(&exp, h->policy->u.mqtt.topic,
+				      strlen(h->policy->u.mqtt.topic),
+				      &used_in, &used_out) != LSTRX_DONE) {
+			lwsl_err("%s, failed to expand MQTT publish"
+				 " topic with no output\n", __func__);
+			return 1;
+		}
+		expbuf = lws_malloc(used_out + 1, __func__);
+		if (!expbuf) {
+			lwsl_err("%s, failed to allocate MQTT publish topic",
+				 __func__);
+			return 1;
+		}
+
+		lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata, expbuf,
+				used_out);
+
+                if (lws_strexp_expand(&exp, h->policy->u.mqtt.topic,
+				      strlen(h->policy->u.mqtt.topic), &used_in,
+				      &used_out) != LSTRX_DONE) {
+			lws_free(expbuf);
+			return 1;
+		}
+		lwsl_notice("%s, expbuf - %s\n", __func__, expbuf);
+		mqpp.topic = (char *)expbuf;
+
+		mqpp.topic_len = (uint16_t)strlen(mqpp.topic);
+		mqpp.packet_id = (uint16_t)(h->txord - 1);
 		mqpp.payload = buf + LWS_PRE;
 		if (h->writeable_len)
-			mqpp.payload_len = h->writeable_len;
+			mqpp.payload_len = (uint32_t)h->writeable_len;
 		else
-			mqpp.payload_len = buflen;
+			mqpp.payload_len = (uint32_t)buflen;
 
 		lwsl_notice("%s: payload len %d\n", __func__,
 				(int)mqpp.payload_len);
@@ -199,15 +315,18 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		mqpp.qos = h->policy->u.mqtt.qos;
 
 		if (lws_mqtt_client_send_publish(wsi, &mqpp,
-						 (const char *)buf + LWS_PRE, buflen,
+						 (const char *)buf + LWS_PRE,
+						 (uint32_t)buflen,
 						 f & LWSSS_FLAG_EOM)) {
 			lwsl_notice("%s: failed to publish\n", __func__);
+			lws_free(expbuf);
 
 			return -1;
 		}
+		lws_free(expbuf);
 
 		return 0;
-
+	}
 	default:
 		break;
 	}
@@ -253,16 +372,74 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 	};
 	size_t used_in, olen[4] = { 0, 0, 0, 0 }, tot = 0;
 	lws_strexp_t exp;
-	char *p, *ps[4];
-	int n;
+	char *ps[4];
+	uint8_t *p = NULL;
+	int n = -1;
+	size_t blen;
+	lws_system_blob_t *b = NULL;
 
 	memset(&ct->ccp, 0, sizeof(ct->ccp));
+	b = lws_system_get_blob(i->context,
+				LWS_SYSBLOB_TYPE_MQTT_CLIENT_ID, 0);
+
+	/* If LWS_SYSBLOB_TYPE_MQTT_CLIENT_ID is set */
+	if (b && (blen = lws_system_blob_get_size(b))) {
+		if (blen > LWS_MQTT_MAX_CIDLEN) {
+			lwsl_err("%s - Client ID too long.\n",
+				 __func__);
+			return -1;
+		}
+		p = (uint8_t *)lws_zalloc(blen+1, __func__);
+		n = lws_system_blob_get(b, p, &blen, 0);
+		if (n) {
+			ct->ccp.client_id = NULL;
+		} else {
+			ct->ccp.client_id = (const char *)p;
+			lwsl_notice("%s - Client ID = %s\n",
+				    __func__, ct->ccp.client_id);
+		}
+	} else {
+		/* Default (Random) client ID */
+		ct->ccp.client_id = NULL;
+	}
+
+	b = lws_system_get_blob(i->context,
+				LWS_SYSBLOB_TYPE_MQTT_USERNAME, 0);
+
+	/* If LWS_SYSBLOB_TYPE_MQTT_USERNAME is set */
+	if (b && (blen = lws_system_blob_get_size(b))) {
+		p = (uint8_t *)lws_zalloc(blen+1, __func__);
+		n = lws_system_blob_get(b, p, &blen, 0);
+		if (n) {
+			ct->ccp.username = NULL;
+		} else {
+			ct->ccp.username = (const char *)p;
+			lwsl_notice("%s - Username ID = %s\n",
+				    __func__, ct->ccp.username);
+		}
+	}
+
+	b = lws_system_get_blob(i->context,
+				LWS_SYSBLOB_TYPE_MQTT_PASSWORD, 0);
+
+	/* If LWS_SYSBLOB_TYPE_MQTT_PASSWORD is set */
+	if (b && (blen = lws_system_blob_get_size(b))) {
+		p = (uint8_t *)lws_zalloc(blen+1, __func__);
+		n = lws_system_blob_get(b, p, &blen, 0);
+		if (n) {
+			ct->ccp.password = NULL;
+		} else {
+			ct->ccp.password = (const char *)p;
+			lwsl_notice("%s - Password ID = %s\n",
+				    __func__, ct->ccp.password);
+		}
+	}
 
-	ct->ccp.client_id		= "lwsMqttClient";
 	ct->ccp.keep_alive		= h->policy->u.mqtt.keep_alive;
 	ct->ccp.clean_start		= h->policy->u.mqtt.clean_start;
 	ct->ccp.will_param.qos		= h->policy->u.mqtt.will_qos;
 	ct->ccp.will_param.retain	= h->policy->u.mqtt.will_retain;
+	ct->ccp.aws_iot			= h->policy->u.mqtt.aws_iot;
 	h->u.mqtt.topic_qos.qos		= h->policy->u.mqtt.qos;
 
 	/*
@@ -310,8 +487,8 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 	p = h->u.mqtt.heap_baggage;
 	for (n = 0; n < (int)LWS_ARRAY_SIZE(sources); n++) {
 		lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata,
-				p, (size_t)-1);
-		ps[n] = p;
+				(char *)p, (size_t)-1);
+		ps[n] = (char *)p;
 		if (lws_strexp_expand(&exp, sources[n], strlen(sources[n]),
 				      &used_in, &olen[n]) != LSTRX_DONE)
 			return 1;
